Package: midfieldr
Title: Tools and Methods for Working with MIDFIELD Data in 'R'
Version: 1.0.3
Authors@R: c(
    person("Richard", "Layton", , "graphdoctor@gmail.com", role = c("cre", "aut", "cph")),
    person("Russell", "Long", , "ralong@purdue.edu", role = c("aut", "cph", "dtm")),
    person("Matthew", "Ohland", , "ohland@purdue.edu", role = c("aut", "cph")),
    person("Marisa", "Orr", role = c("aut", "cph")),
    person("Susan", "Lord", , "slord@sandiego.edu", role = c("aut", "cph"))
  )
Maintainer: Richard Layton <graphdoctor@gmail.com>
Description: Provides tools and demonstrates methods for working with
    individual undergraduate student-level records (registrar's data) in
    'R'. Tools include filters for program codes, data sufficiency, and
    timely completion. Methods include gathering blocs of records,
    computing quantitative metrics such as graduation rate, and creating
    charts to visualize comparisons. 'midfieldr' interacts with practice
    data provided in 'midfielddata', an R data package available at
    <https://midfieldr.github.io/midfielddata/>.  'midfieldr' also
    interacts with the full MIDFIELD database for users who have access.
    As of the transfer of MIDFIELD to the American Society for Engineering
    Education in 2023, the development, expansion, and study of
    MIDFIELD has been supported by the National Science Foundation grants
    0337629, 0646441, 0729596, 0734062, 0835914, 0935157, 0935058,
    0969474, 1025171, 1129383, 1232740, 1329283, 1361058, 1545667,
    2142087, 2141903, and 2152441.
License: MIT + file LICENSE
URL: https://midfieldr.github.io/midfieldr/
BugReports: https://github.com/MIDFIELDR/midfieldr/issues
Depends: R (>= 3.5.0)
Imports: checkmate, data.table (>= 1.9.8), stats, wrapr
Suggests: knitr, rmarkdown, tinytest
VignetteBuilder: knitr, rmarkdown
Config/roxygen2/version: 8.0.0
Encoding: UTF-8
Language: en-US
LazyData: TRUE
LazyDataCompression: bzip2
NeedsCompilation: no
Packaged: 2026-05-10 16:20:22 UTC; layto
Author: Richard Layton [cre, aut, cph],
  Russell Long [aut, cph, dtm],
  Matthew Ohland [aut, cph],
  Marisa Orr [aut, cph],
  Susan Lord [aut, cph]
Repository: CRAN
Date/Publication: 2026-05-10 16:30:02 UTC
Built: R 4.5.2; ; 2026-05-10 17:08:23 UTC; unix

Package: midfieldr
Title: Tools and Methods for Working with MIDFIELD Data in 'R'
Version: 1.0.3
Authors@R: c(
    person("Richard", "Layton", , "graphdoctor@gmail.com", role = c("cre", "aut", "cph")),
    person("Russell", "Long", , "ralong@purdue.edu", role = c("aut", "cph", "dtm")),
    person("Matthew", "Ohland", , "ohland@purdue.edu", role = c("aut", "cph")),
    person("Marisa", "Orr", role = c("aut", "cph")),
    person("Susan", "Lord", , "slord@sandiego.edu", role = c("aut", "cph"))
  )
Maintainer: Richard Layton <graphdoctor@gmail.com>
Description: Provides tools and demonstrates methods for working with
    individual undergraduate student-level records (registrar's data) in
    'R'. Tools include filters for program codes, data sufficiency, and
    timely completion. Methods include gathering blocs of records,
    computing quantitative metrics such as graduation rate, and creating
    charts to visualize comparisons. 'midfieldr' interacts with practice
    data provided in 'midfielddata', an R data package available at
    <https://midfieldr.github.io/midfielddata/>.  'midfieldr' also
    interacts with the full MIDFIELD database for users who have access.
    As of the transfer of MIDFIELD to the American Society for Engineering
    Education in 2023, the development, expansion, and study of
    MIDFIELD has been supported by the National Science Foundation grants
    0337629, 0646441, 0729596, 0734062, 0835914, 0935157, 0935058,
    0969474, 1025171, 1129383, 1232740, 1329283, 1361058, 1545667,
    2142087, 2141903, and 2152441.
License: MIT + file LICENSE
URL: https://midfieldr.github.io/midfieldr/
BugReports: https://github.com/MIDFIELDR/midfieldr/issues
Depends: R (>= 3.5.0)
Imports: checkmate, data.table (>= 1.9.8), stats, wrapr
Suggests: knitr, rmarkdown, tinytest
VignetteBuilder: knitr, rmarkdown
Config/roxygen2/version: 8.0.0
Encoding: UTF-8
Language: en-US
LazyData: TRUE
LazyDataCompression: bzip2
NeedsCompilation: no
Packaged: 2026-05-10 16:20:22 UTC; layto
Author: Richard Layton [cre, aut, cph],
  Russell Long [aut, cph, dtm],
  Matthew Ohland [aut, cph],
  Marisa Orr [aut, cph],
  Susan Lord [aut, cph]
Repository: CRAN
Date/Publication: 2026-05-10 16:30:02 UTC
Built: R 4.6.0; ; 2026-05-10 17:23:54 UTC; unix

# Generated by roxygen2: do not edit by hand

export(add_completion_status)
export(add_data_sufficiency)
export(add_timely_term)
export(check_equiv_frames)
export(filter_cip)
export(order_multiway)
export(prep_fye_mice)
export(select_required)
importFrom(checkmate,assert_data_frame)
importFrom(checkmate,assert_int)
importFrom(checkmate,assert_names)
importFrom(checkmate,assert_subset)
importFrom(checkmate,qassert)
importFrom(data.table,"%chin%")
importFrom(data.table,"%like%")
importFrom(data.table,":=")
importFrom(data.table,.N)
importFrom(data.table,.SD)
importFrom(data.table,as.data.table)
importFrom(data.table,copy)
importFrom(data.table,fcase)
importFrom(data.table,fifelse)
importFrom(data.table,setDF)
importFrom(data.table,setDT)
importFrom(data.table,setcolorder)
importFrom(data.table,setkey)
importFrom(data.table,setkeyv)
importFrom(data.table,setnames)
importFrom(data.table,setorder)
importFrom(data.table,setorderv)
importFrom(data.table,shift)
importFrom(stats,na.omit)
importFrom(stats,reorder)
importFrom(wrapr,"%?%")
importFrom(wrapr,check_equiv_frames)
importFrom(wrapr,let)
importFrom(wrapr,stop_if_dot_args)

<!-- README.md is generated from README.Rmd. Please edit that file -->

<br>midfieldr is an R package that provides tools and methods for
studying undergraduate student-level records from the MIDFIELD database.

<img src="man/figures/logo.png" style="width:15.0%" />

## Introduction

Provides tools and demonstrates methods for working with individual
undergraduate student-level records (registrar’s data) in R. Tools
include filters for program codes, data sufficiency, and timely
completion. Methods include gathering blocs of records, computing
quantitative metrics such as graduation rate, and creating charts to
visualize comparisons. midfieldr is designed to work with data from the
MIDFIELD research database, a sample of which is available in the
midfielddata data package.

midfieldr provides these functions for manipulating student-level data:

- `add_completion_status()` Determine completion status for every
  student
- `add_data_sufficiency()` Determine data sufficiency for every student
- `add_timely_term()` Calculate a timely completion term for every
  student
- `filter_cip()` Filter CIP data to match search strings
- `prep_fye_mice()` Prepare FYE data for multiple imputation
- `select_required()` Select required midfieldr variables

Additional functions for processing intermediate results:

- `order_multiway()` Order categorical variables of multiway data

R packages in examples and vignettes

- *Data preparation.*   We use the data.table system and some base R for
  data manipulation ([Dowle and Srinivasan
  2022](#ref-Dowle+Srinivasan:2022:data.table)). To assist users who
  might prefer other systems, the MIDFIELD Institute website ([Lord et
  al. 2024](#ref-midfieldinstitute:2024)) includes tutorials providing
  side-by-side base R, data.table, and dplyr solutions to common data
  shaping tasks using MIDFIELD practice data.  
- *Charts.*   Our preferred package for charts is ggplot2 ([Wickham
  2016](#ref-Wickham:2016:ggplot2)). The lattice package ([Sarkar
  2008](#ref-Sarkar:2008)) also offers users comprehensive control over
  graphical elements (though our lattice experience is no longer
  current).

## Usage

In this example, we gather all students ever enrolled in Engineering and
summarize their graduation status (in any major), grouping by
race/ethnicity and sex. If you are writing your own script to follow
along, we use these packages in this example:

``` r
library(midfieldr)
library(midfielddata)
library(data.table)
```

Load the practice data. Reduce initial dimensions of data tables.

``` r
# Load the practice data
data(student, term, degree)

# Reduce dimensions of source data tables
student <- select_required(student)
term <- select_required(term)
degree <- select_required(degree)
```

Filter for data sufficiency.

``` r
# Initialize the working data frame
DT <- term[, .(mcid, cip6)]

# Filter observations for data sufficiency
DT <- add_timely_term(DT, term)
DT <- add_data_sufficiency(DT, term)
DT <- DT[data_sufficiency == "include"]
```

Filter for degree-seeking students ever enrolled in Engineering.

``` r
# Inner join to filter observations for degree-seeking
cols_we_want <- student[, .(mcid)]
DT <- cols_we_want[DT, on = c("mcid"), nomatch = NULL]

# Filter observations for engineering programs
DT <- DT[cip6 %like% "^14"]

# Filter observations for unique students (first instance)
DT <- DT[, .SD[1], by = c("mcid")]
```

Determine completion status.

``` r
# Add completion status variable
DT <- add_completion_status(DT, degree)
```

Aggregate observations by groupings.

``` r
# Left join to add race/ethnicity and sex variables (omit unknowns)
cols_we_want <- student[, .(mcid, race, sex)]
DT <- student[DT, on = c("mcid")]
DT <- DT[!(race %ilike% "unknown" | sex %ilike% "unknown")]

# Create a variable combining race/ethnicity and sex
DT[, people := paste(race, sex)]

# Aggregate observations by groupings
DT_display <- DT[, .N, by = c("completion_status", "people")]
setorderv(DT_display, c("completion_status", "people"))
```

Reshape results for display.

``` r
# Transform to row-record form
DT_display <- dcast(DT_display, people ~ completion_status, value.var = "N", fill = 0)

# Prepare the table for display
setcolorder(DT_display, c("people", "timely", "late"))
setkeyv(DT_display, c("people"))
setnames(DT_display,
  old = c("people", "timely", "late", "NA"),
  new = c("People", "Timely completion", "Late completion", "Did not complete")
)
```

Print the results table. “Timely completion” is the count of graduates
completing their programs in no more than 6 years; “Late completion” is
the count of those graduating in more than 6 years; “Did not complete”
is the count of non-graduates.

<div id="oqnunstppn" style="padding-left:0px;padding-right:0px;padding-top:10px;padding-bottom:10px;overflow-x:auto;overflow-y:auto;width:auto;height:auto;">
<style>#oqnunstppn table {
  font-family: system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji';
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}
&#10;#oqnunstppn thead, #oqnunstppn tbody, #oqnunstppn tfoot, #oqnunstppn tr, #oqnunstppn td, #oqnunstppn th {
  border-style: none;
}
&#10;#oqnunstppn p {
  margin: 0;
  padding: 0;
}
&#10;#oqnunstppn .gt_table {
  display: table;
  border-collapse: collapse;
  line-height: normal;
  margin-left: auto;
  margin-right: auto;
  color: #333333;
  font-size: small;
  font-weight: normal;
  font-style: normal;
  background-color: #FFFFFF;
  width: auto;
  border-top-style: solid;
  border-top-width: 2px;
  border-top-color: #000000;
  border-right-style: none;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #000000;
  border-left-style: none;
  border-left-width: 2px;
  border-left-color: #D3D3D3;
}
&#10;#oqnunstppn .gt_caption {
  padding-top: 4px;
  padding-bottom: 4px;
}
&#10;#oqnunstppn .gt_title {
  color: #333333;
  font-size: 125%;
  font-weight: initial;
  padding-top: 4px;
  padding-bottom: 4px;
  padding-left: 5px;
  padding-right: 5px;
  border-bottom-color: #FFFFFF;
  border-bottom-width: 0;
}
&#10;#oqnunstppn .gt_subtitle {
  color: #333333;
  font-size: 85%;
  font-weight: initial;
  padding-top: 3px;
  padding-bottom: 5px;
  padding-left: 5px;
  padding-right: 5px;
  border-top-color: #FFFFFF;
  border-top-width: 0;
}
&#10;#oqnunstppn .gt_heading {
  background-color: #FFFFFF;
  text-align: center;
  border-bottom-color: #FFFFFF;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
}
&#10;#oqnunstppn .gt_bottom_border {
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #5F5F5F;
}
&#10;#oqnunstppn .gt_col_headings {
  border-top-style: solid;
  border-top-width: 2px;
  border-top-color: #5F5F5F;
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #5F5F5F;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
}
&#10;#oqnunstppn .gt_col_heading {
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: normal;
  text-transform: inherit;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
  vertical-align: bottom;
  padding-top: 5px;
  padding-bottom: 6px;
  padding-left: 5px;
  padding-right: 5px;
  overflow-x: hidden;
}
&#10;#oqnunstppn .gt_column_spanner_outer {
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: normal;
  text-transform: inherit;
  padding-top: 0;
  padding-bottom: 0;
  padding-left: 4px;
  padding-right: 4px;
}
&#10;#oqnunstppn .gt_column_spanner_outer:first-child {
  padding-left: 0;
}
&#10;#oqnunstppn .gt_column_spanner_outer:last-child {
  padding-right: 0;
}
&#10;#oqnunstppn .gt_column_spanner {
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #5F5F5F;
  vertical-align: bottom;
  padding-top: 5px;
  padding-bottom: 5px;
  overflow-x: hidden;
  display: inline-block;
  width: 100%;
}
&#10;#oqnunstppn .gt_spanner_row {
  border-bottom-style: hidden;
}
&#10;#oqnunstppn .gt_group_heading {
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: initial;
  text-transform: inherit;
  border-top-style: solid;
  border-top-width: 2px;
  border-top-color: #5F5F5F;
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #5F5F5F;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
  vertical-align: middle;
  text-align: left;
}
&#10;#oqnunstppn .gt_empty_group_heading {
  padding: 0.5px;
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: initial;
  border-top-style: solid;
  border-top-width: 2px;
  border-top-color: #5F5F5F;
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #5F5F5F;
  vertical-align: middle;
}
&#10;#oqnunstppn .gt_from_md > :first-child {
  margin-top: 0;
}
&#10;#oqnunstppn .gt_from_md > :last-child {
  margin-bottom: 0;
}
&#10;#oqnunstppn .gt_row {
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
  margin: 10px;
  border-top-style: none;
  border-top-width: 1px;
  border-top-color: #D5D5D5;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D5D5D5;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D5D5D5;
  vertical-align: middle;
  overflow-x: hidden;
}
&#10;#oqnunstppn .gt_stub {
  color: #FFFFFF;
  background-color: #5F5F5F;
  font-size: 100%;
  font-weight: initial;
  text-transform: inherit;
  border-right-style: solid;
  border-right-width: 2px;
  border-right-color: #5F5F5F;
  padding-left: 5px;
  padding-right: 5px;
}
&#10;#oqnunstppn .gt_stub_row_group {
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: initial;
  text-transform: inherit;
  border-right-style: solid;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
  padding-left: 5px;
  padding-right: 5px;
  vertical-align: top;
}
&#10;#oqnunstppn .gt_row_group_first td {
  border-top-width: 2px;
}
&#10;#oqnunstppn .gt_row_group_first th {
  border-top-width: 2px;
}
&#10;#oqnunstppn .gt_summary_row {
  color: #333333;
  background-color: #FFFFFF;
  text-transform: inherit;
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
}
&#10;#oqnunstppn .gt_first_summary_row {
  border-top-style: solid;
  border-top-color: #5F5F5F;
}
&#10;#oqnunstppn .gt_first_summary_row.thick {
  border-top-width: 2px;
}
&#10;#oqnunstppn .gt_last_summary_row {
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #5F5F5F;
}
&#10;#oqnunstppn .gt_grand_summary_row {
  color: #333333;
  background-color: #D5D5D5;
  text-transform: inherit;
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
}
&#10;#oqnunstppn .gt_first_grand_summary_row {
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
  border-top-style: double;
  border-top-width: 6px;
  border-top-color: #5F5F5F;
}
&#10;#oqnunstppn .gt_last_grand_summary_row_top {
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
  border-bottom-style: double;
  border-bottom-width: 6px;
  border-bottom-color: #5F5F5F;
}
&#10;#oqnunstppn .gt_striped {
  background-color: #F4F4F4;
}
&#10;#oqnunstppn .gt_table_body {
  border-top-style: solid;
  border-top-width: 2px;
  border-top-color: #5F5F5F;
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #5F5F5F;
}
&#10;#oqnunstppn .gt_footnotes {
  color: #333333;
  background-color: #FFFFFF;
  border-bottom-style: none;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
  border-left-style: none;
  border-left-width: 2px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
}
&#10;#oqnunstppn .gt_footnote {
  margin: 0px;
  font-size: 90%;
  padding-top: 4px;
  padding-bottom: 4px;
  padding-left: 5px;
  padding-right: 5px;
}
&#10;#oqnunstppn .gt_sourcenotes {
  color: #333333;
  background-color: #FFFFFF;
  border-bottom-style: none;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
  border-left-style: none;
  border-left-width: 2px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
}
&#10;#oqnunstppn .gt_sourcenote {
  font-size: 90%;
  padding-top: 4px;
  padding-bottom: 4px;
  padding-left: 5px;
  padding-right: 5px;
}
&#10;#oqnunstppn .gt_left {
  text-align: left;
}
&#10;#oqnunstppn .gt_center {
  text-align: center;
}
&#10;#oqnunstppn .gt_right {
  text-align: right;
  font-variant-numeric: tabular-nums;
}
&#10;#oqnunstppn .gt_font_normal {
  font-weight: normal;
}
&#10;#oqnunstppn .gt_font_bold {
  font-weight: bold;
}
&#10;#oqnunstppn .gt_font_italic {
  font-style: italic;
}
&#10;#oqnunstppn .gt_super {
  font-size: 65%;
}
&#10;#oqnunstppn .gt_footnote_marks {
  font-size: 75%;
  vertical-align: 0.4em;
  position: initial;
}
&#10;#oqnunstppn .gt_asterisk {
  font-size: 100%;
  vertical-align: 0;
}
&#10;#oqnunstppn .gt_indent_1 {
  text-indent: 5px;
}
&#10;#oqnunstppn .gt_indent_2 {
  text-indent: 10px;
}
&#10;#oqnunstppn .gt_indent_3 {
  text-indent: 15px;
}
&#10;#oqnunstppn .gt_indent_4 {
  text-indent: 20px;
}
&#10;#oqnunstppn .gt_indent_5 {
  text-indent: 25px;
}
&#10;#oqnunstppn .katex-display {
  display: inline-flex !important;
  margin-bottom: 0.75em !important;
}
&#10;#oqnunstppn div.Reactable > div.rt-table > div.rt-thead > div.rt-tr.rt-tr-group-header > div.rt-th-group:after {
  height: 0px !important;
}
</style>
<table class="gt_table" data-quarto-disable-processing="false" data-quarto-bootstrap="false">
  <caption>Table 1: Completion status of engineering undergraduates in the practice data</caption>
  <thead>
    <tr class="gt_col_headings">
      <th class="gt_col_heading gt_columns_bottom_border gt_left" rowspan="1" colspan="1" style="background-color: #C7EAE5;" scope="col" id="People">People</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="background-color: #C7EAE5;" scope="col" id="Timely-completion">Timely completion</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="background-color: #C7EAE5;" scope="col" id="Late-completion">Late completion</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="background-color: #C7EAE5;" scope="col" id="Did-not-complete">Did not complete</th>
    </tr>
  </thead>
  <tbody class="gt_table_body">
    <tr><td headers="People" class="gt_row gt_left">Asian Female</td>
<td headers="Timely completion" class="gt_row gt_right">87</td>
<td headers="Late completion" class="gt_row gt_right">4</td>
<td headers="Did not complete" class="gt_row gt_right">43</td></tr>
    <tr><td headers="People" class="gt_row gt_left gt_striped">Asian Male</td>
<td headers="Timely completion" class="gt_row gt_right gt_striped">315</td>
<td headers="Late completion" class="gt_row gt_right gt_striped">19</td>
<td headers="Did not complete" class="gt_row gt_right gt_striped">163</td></tr>
    <tr><td headers="People" class="gt_row gt_left">Black Female</td>
<td headers="Timely completion" class="gt_row gt_right">26</td>
<td headers="Late completion" class="gt_row gt_right">3</td>
<td headers="Did not complete" class="gt_row gt_right">39</td></tr>
    <tr><td headers="People" class="gt_row gt_left gt_striped">Black Male</td>
<td headers="Timely completion" class="gt_row gt_right gt_striped">80</td>
<td headers="Late completion" class="gt_row gt_right gt_striped">5</td>
<td headers="Did not complete" class="gt_row gt_right gt_striped">84</td></tr>
    <tr><td headers="People" class="gt_row gt_left">Hispanic Female</td>
<td headers="Timely completion" class="gt_row gt_right">36</td>
<td headers="Late completion" class="gt_row gt_right">3</td>
<td headers="Did not complete" class="gt_row gt_right">31</td></tr>
    <tr><td headers="People" class="gt_row gt_left gt_striped">Hispanic Male</td>
<td headers="Timely completion" class="gt_row gt_right gt_striped">181</td>
<td headers="Late completion" class="gt_row gt_right gt_striped">19</td>
<td headers="Did not complete" class="gt_row gt_right gt_striped">102</td></tr>
    <tr><td headers="People" class="gt_row gt_left">International Female</td>
<td headers="Timely completion" class="gt_row gt_right">110</td>
<td headers="Late completion" class="gt_row gt_right">9</td>
<td headers="Did not complete" class="gt_row gt_right">51</td></tr>
    <tr><td headers="People" class="gt_row gt_left gt_striped">International Male</td>
<td headers="Timely completion" class="gt_row gt_right gt_striped">501</td>
<td headers="Late completion" class="gt_row gt_right gt_striped">41</td>
<td headers="Did not complete" class="gt_row gt_right gt_striped">280</td></tr>
    <tr><td headers="People" class="gt_row gt_left">Native American Female</td>
<td headers="Timely completion" class="gt_row gt_right">2</td>
<td headers="Late completion" class="gt_row gt_right">0</td>
<td headers="Did not complete" class="gt_row gt_right">2</td></tr>
    <tr><td headers="People" class="gt_row gt_left gt_striped">Native American Male</td>
<td headers="Timely completion" class="gt_row gt_right gt_striped">13</td>
<td headers="Late completion" class="gt_row gt_right gt_striped">3</td>
<td headers="Did not complete" class="gt_row gt_right gt_striped">6</td></tr>
    <tr><td headers="People" class="gt_row gt_left">White Female</td>
<td headers="Timely completion" class="gt_row gt_right">985</td>
<td headers="Late completion" class="gt_row gt_right">51</td>
<td headers="Did not complete" class="gt_row gt_right">386</td></tr>
    <tr><td headers="People" class="gt_row gt_left gt_striped">White Male</td>
<td headers="Timely completion" class="gt_row gt_right gt_striped">4100</td>
<td headers="Late completion" class="gt_row gt_right gt_striped">269</td>
<td headers="Did not complete" class="gt_row gt_right gt_striped">2034</td></tr>
  </tbody>
  &#10;</table>
</div>

*Reminder.*   midfielddata is suitable for learning to work with
student-level data but not for drawing inferences about program
attributes or student experiences. midfielddata supplies practice data,
not research data.

## Installation

Install from CRAN with:

``` r
install.packages("midfieldr")
```

You can install the development version of midfieldr from GitHub with:

``` r
# install.packages("pak")
pak::pak("MIDFIELDR/midfieldr")
```

The installed size of midfielddata is about 24 Mb, so the installation
takes some time. Install from the MIDFIELDR drat repository with:

``` r
install.packages("midfielddata",
  repos = "https://MIDFIELDR.github.io/drat/",
  type = "source"
)
```

## More information

[MIDFIELD](https://midfield.online/).   A database of anonymized
student-level records for approximately 2.4M undergraduates at 21 US
institutions from 1987-2022. In 2023, control and management of the
database was transferred to the American Society for Engineering
Education (ASEE). For further information, contact ASEE.

[midfielddata](https://midfieldr.github.io/midfielddata/).   An R data
package that supplies anonymized student-level records for 98,000
undergraduates at three US institutions from 1988-2018. A sample of the
MIDFIELD database, midfielddata provides practice data for the tools and
methods in the midfieldr package.

## Acknowledgments

As of the transfer of MIDFIELD to the American Society for Engineering
Education (ASEE) in 2023, the development, expansion, and study of
MIDFIELD has been supported by the National Science Foundation grants
0337629, 0646441, 0729596, 0734062, 0835914, 0935157, 0935058, 0969474,
1025171, 1129383, 1232740, 1329283, 1361058, 1545667, 2142087, 2141903,
and 2152441.

## References

<div id="refs" class="references csl-bib-body hanging-indent">

<!-- blockquote { -->

<!--     padding:     10px 20px; -->

<!--     margin:      0 0 20px; -->

<!--     border-left: 0px -->

<!-- } -->

<!-- caption { -->

<!--     color:       #525252; -->

<!--     text-align:  left; -->

<!--     font-weight: normal; -->

<!--     font-size:   medium; -->

<!--     line-height: 1.5; -->

<!-- } -->

<div id="ref-Dowle+Srinivasan:2022:data.table" class="csl-entry">

Dowle, Matt, and Arun Srinivasan. 2022.
*<span class="nocase">d</span>ata.table: Extension of ‘Data.frame‘*.
R package version 1.14.6.
<https://CRAN.R-project.org/package=data.table>.

</div>

<div id="ref-midfieldinstitute:2024" class="csl-entry">

Lord, Susan, Richard Layton, Russell Long, Matthew Ohland, and Marisa
Orr. 2024. *MIDFIELD Institute*.
<https://midfieldr.github.io/2024-midfield-institute/>.

</div>

<div id="ref-Sarkar:2008" class="csl-entry">

Sarkar, Deepayan. 2008. *<span class="nocase">lattice: Multivariate Data
Visualization with R</span>*. Springer.
<http://lmdvr.r-forge.r-project.org>.

</div>

<div id="ref-Wickham:2016:ggplot2" class="csl-entry">

Wickham, Hadley. 2016. *<span class="nocase">ggplot2: Elegant Graphics
for Data Analysis</span>*. ISBN 978-3-319-24277-4; Springer-Verlag New
York. <https://ggplot2.tidyverse.org>.

</div>

</div>

# Subset using keywords
filter_cip(keep_text = "engineering")

\donttest{
    # Multiple passes to narrow the results
    first_pass <- filter_cip("civil")
    second_pass <- filter_cip("engineering", cip = first_pass)
    filter_cip(drop_text = "technology", cip = second_pass)
    
    # drop_text argument, when used, must be named
    filter_cip("civil engineering", drop_text = "technology")
    
    # Subset using numerical codes
    filter_cip(keep_text = c("050125", "160501"))
    
    # Subset using regular expressions
    filter_cip(keep_text = "^54")
    filter_cip(keep_text = c("^1407", "^1408"))
    
    # Select columns
    filter_cip(keep_text = "^54", select = c("cip6", "cip4name"))
}

# Documentation described below using an inline R code chunk, e.g.,
# "`r dframe_order_multiway`" or "`r return_order_multiway`", are
# documented in the R/roxygen.R file.


#' Order categorical variables of multiway data
#'
#' Transform a data frame such that two independent categorical variables are
#' factors with levels ordered for display in a multiway dot plot. Multiway data
#' comprise a single quantitative value (or response) for every combination of
#' levels of two categorical variables. The ordering of the rows and panels is
#' crucial to the perception of effects (Cleveland, 1993).
#'
#' In our context, "multiway" refers to the data structure and graph design
#' defined by Cleveland (1993), not to the methods of analysis described by
#' Kroonenberg (2008).
#'
#' Multiway data comprise three variables: a categorical variable of _m_ levels;
#' a second independent categorical variable of _n_ levels; and a quantitative
#' variable (or _response_) of length _mn_ that cross-classifies the categories,
#' that is, there is a value of the response for each combination of levels of
#' the two categorical variables.
#'
#' In a multiway dot plot, one category is encoded by the panels, the second
#' category is encoded by the rows of each panel, and the quantitative variable
#' is encoded along identical horizontal scales.
#'
#' @param dframe  `r dframe_order_multiway`
#' @param quantity Character, name (in quotes) of the single multiway quantitative
#'        variable
#' @param categories Character, vector of names (in quotes) of the two multiway
#'        categorical variables
#' @param ... `r param_dots`
#' @param method Character, “median” (default) or “percent”, method of
#'        ordering the levels of the categories. The median method computes the
#'        medians of the quantitative column grouped by category. The percent
#'        method computes percentages based on the same ratio underlying the
#'        quantitative percentage variable except grouped by category.
#' @param ratio_of Character vector with the names (in quotes) of the
#'        numerator and denominator columns that produced the quantitative
#'        variable, required when `method` is "percent". Names can be
#'        in any order; the algorithm assumes that the parameter with the
#'        larger column sum is the denominator of the ratio.
#'
#' @return `r return_order_multiway`
#' \describe{
#'  \item{`CATEGORY_median` columns (when ordering method is "median")}{
#'      Numeric. Two columns of medians of the quantitative variable grouped
#'      by the categorical variables. The `CATEGORY` placeholder in
#'      the column name is replaced by a category name from the
#'      `categories` argument. For example, suppose
#'      `categories = c("program", "people")` and
#'      `method = "median"`. The two new column names would be
#'      `program_median` and `people_median.`}
#'
#'  \item{`CATEGORY_QUANTITY` columns (when ordering method is "percent")}{
#'      Numeric. Two columns of percentages based on the same ratio that
#'      produces the quantitative variable except grouped by the categorical
#'      variables. The `CATEGORY` placeholder in the column name is
#'      replaced by a category name from the `categories` argument; the
#'      `QUANTITY` placeholder is replaced by the quantitative variable
#'      name in the `quantity` argument. For example, suppose
#'      `categories = c("program", "people")`, and
#'      `quantity = "grad_rate"`, and `method = "percent"`. The two
#'      new column names  would be `program_grad_rate` and
#'      `people_grad_rate.`}
#'
#' }
#'
#' @references
#'   Cleveland WS (1993). \emph{Visualizing Data}. Hobart Press, Summit, NJ.
#'
#'   Kroonenberg PM (2008). \emph{Applied Multiway Data Analysis}. Wiley,
#'   Hoboken, NJ.
#'
#' @example man/examples/order_multiway_exa.R
#' @export
#'
order_multiway <- function(dframe,
                           quantity,
                           categories,
                           ...,
                           method = NULL,
                           ratio_of = NULL) {
  on.exit(setkey(DT, NULL), add = TRUE)

  # assert arguments after dots used by name
  wrapr::stop_if_dot_args(
    substitute(list(...)),
    paste(
      "Arguments after ... must be named.\n",
      "* Did you forget to write `method = `\n",
      "* or `ratio_of = ` ?\n"
    )
  )

  # required data frame(s)
  qassert(dframe, "d+") # data frame, missing values OK, length 1 or more
  DT <- copy(dframe)
  setDT(DT)

  # required columns
  assert_names(colnames(DT), must.include = c(quantity, categories))

  # class of required columns
  qassert(DT[[quantity]], "n+") # numeric, length 1 or more

  col_class <- DT[, categories, with = FALSE]
  col_class <- unlist(lapply(col_class, class))
  assert_subset(
    col_class,
    choices = c("character", "factor"),
    empty.ok = FALSE,
    .var.name = "categories"
  )

  # other required arguments
  qassert(quantity, "S1") # string, missing values prohibited, length 1
  qassert(categories, "S2") # string, missing values prohibited, length 2

  # optional arguments
  method <- method %?% "median"
  qassert(method, "S1")
  assert_subset(
    method,
    # choices = c("median", "mean", "sum", "percent", "alphabet"),
    choices = c("median", "percent"),
    empty.ok = FALSE,
    .var.name = "method"
  )

  if (method == "percent") {
    qassert(ratio_of, "S2")
    assert_subset(
      ratio_of,
      choices = names(DT),
      empty.ok = FALSE,
      .var.name = "ratio_of"
    )
    # columns must be numeric
    col_class <- DT[, ratio_of, with = FALSE]
    col_class <- unlist(lapply(col_class, class))
    checkmate::assert_subset(
      col_class,
      choices = c("numeric", "double", "integer"),
      empty.ok = FALSE,
      .var.name = "ratio_of"
    )
  } else {
    if (!is.null(ratio_of)) {
      warning("Unused argument 'ratio_of'.")
    }
  }

  # do the work

  # all methods treat categories as factors
  DT[, (categories) := lapply(.SD, as.factor), .SDcols = categories]
  # and quantity double, not integer
  DT[, (quantity) := lapply(.SD, as.double), .SDcols = quantity]


  # multiway must have two and only two categories
  categ_1 <- categories[[1]]
  categ_2 <- categories[[2]]

  # if (method == "alphabet") {
  #
  #   # alphabetical order returns categories as factors with levels
  #   # ordered in reverse alphabetical order such that the graph rows and
  #   # panels are in alphabetical order from the top down
  #   DT <- order_by_alphabet(
  #     DT,
  #     categ_1,
  #     categ_2
  #   )
  #
  #   # organize the return column order
  #   setcolorder(DT, c(categories, quantity))
  # } else

  if (method == "percent") {
    # ratio-based metrics, e.g., stickiness or graduation rate
    # returns categories as factors with levels ordered by the metric
    DT <- order_by_percent(
      DT,
      categories,
      quantity,
      ratio_of
    )

    # organize the return column order
    setcolorder(DT, c(categories, ratio_of, quantity))
  } else { # method = "median"

    # function-based ordering returns categories as factors with levels
    # ordered by category median()
    DT <- order_by_function(
      DT,
      categ_1,
      categ_2,
      quantity,
      method
    )

    # organize the return column order
    setcolorder(DT, c(categories, quantity))
  }
  DT[]
}
# --------------------------------------------------------------------------
# internal functions
# --------------------------------------------------------------------------
order_by_percent <- function(dframe,
                             categories,
                             quantity,
                             ratio_of) {
  # avoid possible copy-by-reference side effects
  DT <- copy(dframe)

  # bind names due to NSE notes in R CMD check
  A <- NULL
  B <- NULL
  CATEG_I <- NULL
  COUNT_I <- NULL
  NEW_COL <- NULL

  # replace NA in count columns with zero
  DT[, (ratio_of) := lapply(.SD, function(quantity) {
    fifelse(is.na(quantity), 0, quantity)
  }), .SDcols = ratio_of]

  # sum the two counts by the individual categories
  # provides columns needed to determine row and panel order
  for (categ_i in categories) {
    for (count_i in ratio_of) {
      # execute expressions with column names as parameters
      wrapr::let(
        c(COUNT_I = count_i),
        {
          new_col <- paste(categ_i, count_i, sep = "_")
          DT[, (new_col) := sum(COUNT_I), by = categ_i]
        }
      )
    }
  }

  # Determine the names of the columns used as the numerator and
  # denominator of the ratio. Assumes the smaller number is the numerator,
  # e.g., grad / ever or grad / start. Always more starters or ever-enrolled
  # overall (summing across all programs) than grads.
  count_col_totals <- colSums(DT[, ratio_of, with = FALSE])
  count_col_min <- names(which.min(count_col_totals))
  count_col_max <- names(which.max(count_col_totals))

  # computing the metric for individual categories
  # used for ordering rows and panels
  for (categ_i in categories) {
    # names of new columns, numerator and denominator of
    # category summary metric
    a <- paste(categ_i, count_col_min, sep = "_")
    b <- paste(categ_i, count_col_max, sep = "_")
    new_col <- paste(categ_i, quantity, sep = "_")

    # execute expressions with column names as parameters
    wrapr::let(
      c(
        A = a,
        B = b,
        CATEG_I = categ_i,
        NEW_COL = new_col
      ),
      {
        # percent-based metric by individual category
        DT[, NEW_COL := round(100 * A / B, 1)]

        # order factor levels by values in new column
        DT[, CATEG_I := reorder(CATEG_I, NEW_COL)]

        # drop A and B with the intermediate sums
        DT[, A := NULL]
        DT[, B := NULL]
      }
    )
  }
  DT[]
}
# --------------------------------------------------------------------------
# order_by_alphabet <- function(dframe,
#                               categ_1,
#                               categ_2) {
#
#   # avoid possible copy-by-reference side effects
#   DT <- copy(dframe)
#
#   # bind names due to NSE notes in R CMD check
#   CATEG_1 <- NULL
#   CATEG_2 <- NULL
#
#   # execute expressions with column names as parameters
#   wrapr::let(
#     c(
#       CATEG_1 = categ_1,
#       CATEG_2 = categ_2
#     ),
#     {
#         setDT(DT)
#         DT[, CATEG_1 := as.character(CATEG_1)]
#         DT[, CATEG_2 := as.character(CATEG_2)]
#         setorder(DT, -CATEG_1, -CATEG_2)
#         DT[, CATEG_1 := as.factor(CATEG_1)]
#         DT[, CATEG_2 := as.factor(CATEG_2)]
#     }
#   )
#   DT[]
# }
# --------------------------------------------------------------------------
order_by_function <- function(dframe,
                              categ_1,
                              categ_2,
                              quantity,
                              method) {
  # avoid possible copy-by-reference side effects
  DT <- copy(dframe)

  # bind names due to NSE notes in R CMD check
  CATEG_1 <- NULL
  CATEG_2 <- NULL
  ORDER_1 <- NULL
  ORDER_2 <- NULL
  QUANTITY <- NULL

  # select function to apply
  if (method == "median") {
    f <- stats::median
  }
  # if (method == "sum") {
  #   f <- sum
  # }
  # if (method == "mean") {
  #   f <- mean
  # }

  # create names for value variables
  order_1 <- paste(categ_1, method, sep = "_")
  order_2 <- paste(categ_2, method, sep = "_")

  # execute expressions with column names as parameters
  wrapr::let(
    c(
      CATEG_1 = categ_1,
      CATEG_2 = categ_2,
      ORDER_1 = order_1,
      ORDER_2 = order_2,
      QUANTITY = quantity
    ),
    {
      DT[, ORDER_1 := f(QUANTITY, na.rm = TRUE), by = CATEG_1]
      DT[, ORDER_2 := f(QUANTITY, na.rm = TRUE), by = CATEG_2]
      DT[, CATEG_1 := reorder(CATEG_1, ORDER_1)]
      DT[, CATEG_2 := reorder(CATEG_2, ORDER_2)]
    }
  )
  DT[]
}

test_order_multiway <- function() {
    
    # usage 
    # order_multiway(dframe,
    #              quantity,
    #              categories,
    #              ..., 
    #              method = NULL,
    #              ratio_of = NULL)
    
    # when running internal checks 
    # library("tinytest")
    
    # Needed for tinytest::build_install_test()
    library("data.table")
    options(
        datatable.print.nrows = 8,
        datatable.print.topn = 8,
        datatable.print.class = TRUE
    )
    
    # create a multiway
    dframe <- wrapr::build_frame(
        "catg1"    , "catg2", "a", "b"   |
            "urban"  , "men"  , 112  , 153 |
            "urban"  , "women", 214  , 1267 |
            "rural"  , "men"  , 310  , 361 |
            "rural"  , "women", 420  , 1472 |
            "suburb" , "men"  , 513  , 549 |
            "suburb" , "women", 615  , 1646 |
            "village", "men"  , 732  , 766 |
            "village", "women", 814  , 1881)
    dframe$metric <- round(dframe$a / dframe$b, 2)
    DT <- copy(dframe)
    setDT(DT)
    
    # apply the conditioning function
    mw_med <- order_multiway(DT, 
                                 quantity = "a", 
                                 categories = c("catg1", "catg2"), 
                                 method = "median")
    mw_pct <- order_multiway(DT, 
                                 quantity = "metric", 
                                 categories = c("catg1", "catg2"), 
                                 method = "percent", 
                                 ratio_of = c("a", "b"))
    
    # Begin tests
    # input can be data.frame or data.table
    expect_equivalent(
        mw_med, 
        order_multiway(dframe,
                           quantity = "a", 
                           categories = c("catg1", "catg2"), 
                           method = "median")
    )
    
    # categories can be characters or factors
    expect_equivalent(
        order_multiway(DT[, .(catg1, catg2, a)],
                           quantity = "a", 
                           categories = c("catg1", "catg2"), 
                           method = "median"), 
        order_multiway(mw_med[, .(catg1, catg2, a)],
                           quantity = "a", 
                           categories = c("catg1", "catg2"), 
                           method = "median")
    )
    
    # overwrites default median columns
    expect_equivalent(
        order_multiway(DT,
                           quantity = "a", 
                           categories = c("catg1", "catg2"), 
                           method = "median"), 
        order_multiway(mw_med,
                           quantity = "a", 
                           categories = c("catg1", "catg2"), 
                           method = "median")
    )
    
    # columns have expected class
    expect_equal(class(mw_med[, catg1]), "factor")
    expect_equal(class(mw_med[, catg2]), "factor")
    expect_equal(class(mw_med[, a]), "numeric")
    expect_equal(class(mw_med[, b]), "numeric")
    expect_equal(class(mw_med[, metric]), "numeric")
    
    # error when input arguments wrong class, NA, or NULL
    p <- "a"
    q <- c("catg1", "catg2")
    expect_error(order_multiway(as.list(dframe), p, q))
    expect_error(order_multiway(dframe, as.list(p), q))
    expect_error(order_multiway(dframe, p, as.list(q)))
    expect_error(order_multiway(NULL, p, q))
    expect_error(order_multiway(dframe, NULL, q))
    expect_error(order_multiway(dframe, p, NULL))
    expect_error(order_multiway(NA, p, q))
    
    # names specified in arguments are columns in dframe
    p <- "a"
    q <- c("catg1", "catg2")
    expect_error(order_multiway(dframe, p, c("catg1", NA_character_)))
    expect_error(order_multiway(dframe, NA_character_, q))
    
    # arguments after ... must be named
    p <- "metric"
    q <- c("catg1", "catg2")
    expect_error(order_multiway(dframe,
                                    p,
                                    q,
                                    NULL,
                                    ratio_of = NULL))
    expect_error(order_multiway(dframe,
                                    p, 
                                    q,
                                    method = NULL,
                                    NULL))
    
    # percent method requires ratio_of
    p <- "metric"
    q <- c("catg1", "catg2")
    expect_error( 
        order_multiway(dframe,
                           p,
                           q,
                           method = "percent",
                           ratio_of = NULL)
    )
    # ratio_of must be numeric
    expect_error( 
        order_multiway(dframe,
                           p,
                           q,
                           method = "percent",
                           ratio_of = c("catg1", "catg2"))
    )
    
    # optional arguments. NULL method same as median
    p <- "a"
    q <- c("catg1", "catg2")
    expect_equal(
        order_multiway(dframe,
                           p,
                           q,
                           method = "median",
                           ratio_of = NULL),
        order_multiway(dframe,
                           p,
                           q,
                           method = NULL,
                           ratio_of = NULL)
    )
    
    # median method produces correct answers
    temp <- DT[, lapply(.SD, median), .SDcols = c("a"), by = c("catg1")]
    setnames(temp, "a", "catg1_median")
    expect_equal(
        temp[, .(catg1_median)], 
        unique(mw_med[, .(catg1_median)])
    )
    temp <- DT[, lapply(.SD, median), .SDcols = c("a"), by = c("catg2")]
    setnames(temp, "a", "catg2_median")
    expect_equal(
        temp[, .(catg2_median)], 
        unique(mw_med[, .(catg2_median)])
    )
    
    # percent method produces correct answers 
    # (order_multiway rounds to one place)
    temp <- DT[, lapply(.SD, sum), .SDcols = c("a", "b"), by = c("catg1")]
    temp[, catg1_metric := round(100 * a / b, 1)]
    expect_equal(
        temp[, .(catg1_metric)], 
        unique(mw_pct[, .(catg1_metric)])
    )
    temp <- DT[, lapply(.SD, sum), .SDcols = c("a", "b"), by = c("catg2")]
    temp[, catg2_metric := round(100 * a / b, 1)]
    expect_equal(
        temp[, .(catg2_metric)], 
        unique(mw_pct[, .(catg2_metric)])
    )

    # warning when ratio_of but method is not "percent" 
    p <- "metric"
    q <- c("catg1", "catg2")
    expect_warning(order_multiway(dframe, 
                                      p, 
                                      q, 
                                      method = "median",
                                      ratio_of = c("a", "b")))
    
    # integer metrics made double
    p <- "a"
    q <- c("catg1", "catg2")
    temp <- copy(DT)
    temp[, a := as.integer(a)]
    temp <- order_multiway(temp, p, q)
    expect_equal(class(temp$a), "numeric")

    # ordering factors does not affect numeric columns
    # inner join to check results
    # median method
    u <- order_multiway(dframe, 
                            quantity = "a", 
                            categories = c("catg1", "catg2"))
    u <- u[, .(catg1, catg2, a)]
    u[, `:=`(catg1 = as.character(catg1), catg2 = as.character(catg2))]
    v <- u[dframe, .(catg1, catg2, a), on = c("catg1", "catg2"), nomatch = NULL]
    expect_equal(u, v)
    
    # percent method
    u <- order_multiway(dframe, 
                            quantity = "metric", 
                            categories = c("catg1", "catg2"), 
                            method = "percent", 
                            ratio_of = c("a", "b"))
    u <- u[, .(catg1, catg2, metric)]
    u[, `:=`(catg1 = as.character(catg1), catg2 = as.character(catg2))]
    v <- u[dframe, .(catg1, catg2, metric), on = c("catg1", "catg2"), nomatch = NULL]
    expect_equal(u, v)

    invisible(NULL)
}

test_order_multiway()

# Documentation described below using an inline R code chunk, e.g.,
# "`r dframe_add_data_sufficiency`" or "`r return_data_frame`", are
# documented in the R/roxygen.R file.


#' Determine data sufficiency for every student
#'
#' Add a column to a data frame of student-level records that labels each row
#' for inclusion or exclusion based on data sufficiency near the upper and lower
#' bounds of an institution's data range.
#'
#' The time span of MIDFIELD term data varies by institution, each having their
#' own lower and upper bounds. For some student records, being at or near these
#' bounds creates unavoidable ambiguity when trying to assess degree completion.
#' Such records must be identified and in most cases excluded to prevent false
#' summary counts.
#'
#' @param dframe        `r dframe_add_data_sufficiency`
#' @param midfield_term `r midfield_term_add_data_sufficiency`
#'
#' @return `r return_data_frame`
#' \describe{
#'  \item{`term_i`}{Character. Initial term of a student's longitudinal
#'  record, encoded YYYYT. Not overwritten if present in `dframe.`}
#'  \item{`lower_limit`}{Character. Initial term of an institution's data
#'  range, encoded YYYYT}
#'  \item{`upper_limit`}{Character. Final term of an institution's data
#'  range, encoded YYYYT}
#'  \item{`data_sufficiency`}{Character. Label each observation for
#'  inclusion or exclusion based on data sufficiency. Possible values are:
#'  `include`, indicating that available data are sufficient for
#'  estimating timely completion; `exclude-upper`, indicating
#'  that data are insufficient at the upper limit of a data range; and
#'  `exclude`-lower, indicating that data are insufficient at the
#'  lower limit.}
#' }
#'
#' @family add_*
#' @example man/examples/add_data_sufficiency_exa.R
#' @export
#'
add_data_sufficiency <- function(dframe, midfield_term = term) {
  # remove keys if any
  on.exit(setkey(dframe, NULL))
  on.exit(setkey(midfield_term, NULL), add = TRUE)

  # required arguments
  qassert(dframe, "d+")
  qassert(midfield_term, "d+")

  # ensure data.table format, changes by reference
  setDT(dframe)
  setDT(midfield_term)

  # required columns
  assert_names(colnames(dframe),
    must.include = c("mcid", "timely_term")
  )
  assert_names(colnames(midfield_term),
    must.include = c("mcid", "institution", "term")
  )

  # class of required columns
  qassert(dframe[, mcid], "s+")
  qassert(dframe[, timely_term], "s+")
  qassert(midfield_term[, mcid], "s+")
  qassert(midfield_term[, institution], "s+")
  qassert(midfield_term[, term], "s+")

  # bind names due to NSE notes in R CMD check
  data_sufficiency <- NULL
  timely_term <- NULL
  upper_limit <- NULL
  lower_limit <- NULL
  term_i <- NULL

  # do the work

  # variables added by this function and functions called (if any)
  inst_limits_cols <- c("lower_limit", "upper_limit")
  new_cols <- c("term_i", inst_limits_cols, "data_sufficiency")

  # retain original variables NOT in the vector of new columns
  old_cols <- find_old_cols(dframe, new_cols)
  dframe <- dframe[, .SD, .SDcols = old_cols]

  # begin
  DT <- copy(dframe)

  # add initial term term_i
  DT <- add_initial_term(DT, midfield_term)

  # obtain lower and upper institution data limits
  DT <- add_inst_limits(DT, midfield_term)

  # default is include
  DT[, data_sufficiency := "include"]

  # exclude if TC term exceeds upper limit
  DT <- DT[timely_term > upper_limit, data_sufficiency := "exclude-upper"]

  # exclude if term_i == lower_limit
  DT <- DT[term_i == lower_limit, data_sufficiency := "exclude-lower"]

  # remove all but essential variables
  DT <- DT[, .SD, .SDcols = c("mcid", new_cols)]

  # ensure no duplicate rows
  setkeyv(DT, "mcid")
  DT <- DT[, .SD[1], by = "mcid"]

  # left join new columns to dframe by key(s)
  setkeyv(dframe, "mcid")
  dframe <- DT[dframe]

  # select columns to return
  final_cols <- c(old_cols, new_cols)
  dframe <- dframe[, .SD, .SDcols = final_cols]

  # old columns as keys, order columns and rows
  set_colrow_order(dframe, old_cols)

  # enable printing (see data.table FAQ 2.23)
  dframe[]
}

# ------------------------------------------------------------------------

# Add upper and lower limits of institution data range

add_inst_limits <- function(dframe, midfield_term) {
  # remove keys if any
  on.exit(setkey(dframe, NULL))
  on.exit(setkey(midfield_term, NULL), add = TRUE)

  # ensure data.table format, changes by reference
  setDT(dframe)
  setDT(midfield_term)

  # add_institution() in utils.R
  if (!"institution" %chin% names(dframe)) {
    dframe <- add_institution(dframe, midfield_term = midfield_term)
  }

  # variables added by this function
  new_cols <- c("lower_limit", "upper_limit")

  # retain original variables NOT in the vector of new columns
  old_cols <- find_old_cols(dframe, new_cols)
  dframe <- dframe[, .SD, .SDcols = old_cols]

  # obtain new_cols keyed by institution,
  cols_we_want <- c("institution", "term")
  DT <- midfield_term[, .SD, .SDcols = cols_we_want]
  DT <- DT[, list(
    lower_limit = min(term),
    upper_limit = max(term)
  ), by = "institution"]

  # ensure no duplicate rows
  DT <- unique(DT)

  # left join new columns to dframe by key(s)
  setkeyv(DT, "institution")
  setkeyv(dframe, "institution")
  dframe <- DT[dframe]

  # enable printing (see data.table FAQ 2.23)
  dframe[]
}

test_add_data_sufficiency <- function() {

    # usage
    # add_data_sufficiency(dframe, midfield_term = term)

    # Needed for tinytest::build_install_test()
    library("data.table")

    # create answers
    library(midfieldr)
    dframe <- toy_student[1:10, .(mcid)]
    dframe <- add_timely_term(dframe, midfield_term = toy_term)
    dframe <- add_data_sufficiency(dframe, midfield_term = toy_term)
    # cat(wrapr::draw_frame(dframe))

    DT <- wrapr::build_frame(
        "mcid"         , "level_i"     , "adj_span", "timely_term", "term_i", "lower_limit", "upper_limit", "data_sufficiency" |
            "MID25784187", "01 Freshman" , 6         , "19943"      , "19885" , "19881"      , "19995"      , "include"          |
            "MID25784974", "02 Sophomore", 5         , "19931"      , "19883" , "19881"      , "19995"      , "include"          |
            "MID25816209", "02 Sophomore", 5         , "19923"      , "19881" , "19881"      , "19995"      , "exclude-lower"    |
            "MID25819358", "02 Sophomore", 5         , "19993"      , "19946" , "19881"      , "19995"      , "include"          |
            "MID25828870", "01 Freshman" , 6         , "19933"      , "19881" , "19881"      , "19995"      , "exclude-lower"    |
            "MID25829749", "03 Junior"   , 4         , "20033"      , "19995" , "19881"      , "19995"      , "exclude-upper"    |
            "MID25841418", "03 Junior"   , 4         , "20013"      , "19981" , "19881"      , "19995"      , "exclude-upper"    |
            "MID25845197", "03 Junior"   , 4         , "19943"      , "19905" , "19881"      , "19995"      , "include"          |
            "MID25846316", "01 Freshman" , 6         , "19963"      , "19911" , "19881"      , "19995"      , "include"          |
            "MID25847220", "01 Freshman" , 6         , "19943"      , "19891" , "19881"      , "19995"      , "include"          )
    setDT(DT)

    # correct answers
    expect_equal(
        DT,
        add_data_sufficiency(dframe, midfield_term = toy_term)
    )

    # overwrites existing columns
    expect_equal(
        DT,
        add_data_sufficiency(DT, midfield_term = toy_term)
    )

    # midfield_term argument must be term or equivalent
    expect_error(
        add_data_sufficiency(DT, midfield_term = toy_student)
        )

    # dframe argument requires institution and timely_term columns
    col_missing <- copy(DT)[, timely_term := NULL]
    expect_error(
        add_data_sufficiency(col_missing, midfield_term = toy_term)
        )

    # midfield_term argument requires institution and term columns
    col_missing <- copy(toy_term)[, institution := NULL]
    expect_error(
        add_data_sufficiency(DT, midfield_term = col_missing)
        )
    col_missing <- copy(toy_term)[, term := NULL]
    expect_error(
        add_data_sufficiency(DT, midfield_term = col_missing)
        )

    # data frame arguments must be data frames
    expect_error(
        add_data_sufficiency(DT$mcid, toy_term)
        )
    expect_error(
        add_data_sufficiency(DT, toy_term$term)
        )

    # required arguments must be explicit
    expect_error(
        add_data_sufficiency(NULL, toy_term)
        )
    expect_error(
        add_data_sufficiency(DT, NULL)
        )

    invisible(NULL)
}

test_add_data_sufficiency()

# Default character vector for selecting columns
default_cols<- c("mcid", "institution", "race", "sex", "^term", "cip6", "level")

# Create one string separated by OR
search_pattern <- paste(default_cols, collapse = "|")

# Find names of columns matching or partially matching 
x <- select_required(toy_student) 
names(x)
grepl(search_pattern, names(x))

x <- select_required(toy_term) 
names(x)
grepl(search_pattern, names(x))

x <- select_required(toy_degree) 
names(x)
grepl(search_pattern, names(x))

x <- select_required(toy_course) 
names(x)
grepl(search_pattern, names(x))

# Adding search terms
x <- select_required(toy_course, select_add = c("abbrev", "number", "grade")) 
names(x)

# Same information and ignore row order, column order
x <- toy_student[order(mcid), .(mcid, institution)]
y <- toy_student[order(institution), .(institution, mcid)]
same_content(x, y)

# Different number of rows
x <- toy_student[1:10]
y <- toy_student[1:11]
same_content(x, y)

# Different column names
x <- toy_student[, .(mcid)]
y <- toy_student[, .(institution)]
same_content(x, y)

# Different number of columns and column names
x <- toy_student[, .(mcid)]
y <- toy_student[, .(mcid, institution)]
same_content(x, y)

# Different number of rows, number of columns, and column names
x <- toy_student
y <- toy_term
same_content(x, y)

# Different row content
x <- toy_student[1:10]
y <- toy_student[2:11]
same_content(x, y)

#' @keywords internal
"_PACKAGE"

#' @importFrom checkmate assert_data_frame
#' @importFrom checkmate assert_int
#' @importFrom checkmate assert_names
#' @importFrom checkmate qassert
#' @importFrom checkmate assert_subset
#' @importFrom data.table setkey
#' @importFrom data.table setkeyv
#' @importFrom data.table setcolorder
#' @importFrom data.table setnames
#' @importFrom data.table setorderv
#' @importFrom data.table setorder
#' @importFrom data.table fcase
#' @importFrom data.table fifelse
#' @importFrom data.table copy
#' @importFrom data.table as.data.table
#' @importFrom data.table shift
#' @importFrom data.table setDF
#' @importFrom data.table setDT
#' @importFrom data.table %chin%
#' @importFrom data.table %like%
#' @importFrom data.table .SD
#' @importFrom data.table .N
#' @importFrom data.table :=
#' @importFrom stats na.omit
#' @importFrom stats reorder
#' @importFrom wrapr let
#' @importFrom wrapr stop_if_dot_args
#' @importFrom wrapr %?%
#' @importFrom wrapr check_equiv_frames
NULL

# bind names due to NSE notes in R CMD check
if (getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    ".",
    # midfield data table names
    "student", "term", "course", "degree",
    # midfielddata column names
    "abbrev", "act_comp", "age_desc", "cip6", "coop", "course", "degree",
    "discipline_midfield", "faculty_rank", "gpa_cumul", "gpa_term", "grade",
    "high_school", "home_zip", "hours_course", "hours_cumul",
    "hours_cumul_attempt", "hours_term", "hours_term_attempt", "hours_transfer",
    "institution", "level", "mcid", "number", "race", "sat_math", "sat_verbal",
    "section", "sex", "standing", "term", "term_course", "term_degree",
    "transfer", "type", "us_citizen"
  ))
}

# Subset of built-in data set
dframe <- study_results[program == "EE" | program == "ME"]
dframe[, people := paste(race, sex)]
dframe[, c("race", "sex") := NULL]
data.table::setcolorder(dframe, c("program", "people"))

# Class before ordering
class(dframe$program)
class(dframe$people)

# Class and levels after ordering
mw1 <- order_multiway(dframe, 
                      quantity = "stickiness", 
                      categories = c("program", "people"))
class(mw1$program)
levels(mw1$program)
class(mw1$people)
levels(mw1$people)

# Display category medians 
mw1

# Existing factors (if any) are re-ordered
mw2 <- dframe
mw2$program <- factor(mw2$program, levels = c("ME", "EE"))

# Levels before conditioning
levels(mw2$program) 

# Levels after conditioning
mw2 <- order_multiway(dframe, 
                      quantity = "stickiness", 
                      categories = c("program", "people"))
levels(mw2$program) 

# Ordering using percent method
order_multiway(dframe, 
               quantity = "stickiness", 
               categories = c("program", "people"), 
               method = "percent", 
               ratio_of = c("graduates", "ever_enrolled"))

# Internal utility functions and re-exports

#' @export
#' @importFrom wrapr check_equiv_frames
wrapr::check_equiv_frames


# ------------------------------------------------------------------------
#
#' Set column order and row order
#'
#' Use the vector of column names in `cols` as the ordering argument in
#' `data.table::setcolorder()`` and as the key argument in
#' `data.table::setkeyv()` to order the rows.
#'
#' @param dframe data frame
#' @param cols character vector of column names to use as keys
#' @noRd
#'
set_colrow_order <- function(dframe, cols) {
  on.exit(setkey(dframe, NULL))

  # ensure dframe is data.table class
  setDT(dframe)

  # column order of data frame by vector of names
  setcolorder(dframe, neworder = cols)

  # order rows by using names as keys
  setkeyv(dframe, cols = cols)
}

# ------------------------------------------------------------------------
#
#' Return column names not overwritten by the function
#'
#' Identifies the names of columns unaffected by the function operation.
#' Used by several midfieldr "add_" functions.
#'
#' @param dframe data frame
#' @param new_cols character vector of column names added by the function
#' @noRd
#'
find_old_cols <- function(dframe, new_cols) {
  all_cols <- colnames(dframe)
  old_cols <- all_cols[!all_cols %chin% new_cols]
  return(old_cols)
}


# ------------------------------------------------------------------------
#
#' Add a column of institution names
#'
#' Add a column of character values with institution names (or labels) using
#' student ID as the join-by variable. Obtains the information from the MIDFIELD
#' `term` data table or equivalent. In the MIDFIELD practice data, the labels
#' are de-identified.
#'
#' If a student is associated with more than one institution, the institution at
#' which they completed the most terms is returned. An existing column with the
#' same name as the added column is overwritten.
#'
#' @param dframe Data frame with required variable `mcid.`
#' @param midfield_term MIDFIELD `term` data table or equivalent with required
#' variables `mcid`, `institution`, and `term`.
#' @noRd
#'
add_institution <- function(dframe,
                            midfield_term = term) {
  # remove all keys
  on.exit(setkey(dframe, NULL))
  on.exit(setkey(midfield_term, NULL), add = TRUE)

  # required arguments
  qassert(dframe, "d+")
  qassert(midfield_term, "d+")

  # optional arguments
  # NA

  # inputs modified (or not) by reference
  dframe <- copy(as.data.table(dframe)) #  must copy
  setDT(midfield_term) # immediately subset, so side-effect OK

  # required columns
  assert_names(colnames(dframe),
    must.include = c("mcid")
  )
  assert_names(colnames(midfield_term),
    must.include = c("mcid", "institution", "term")
  )

  # class of required columns
  qassert(dframe[, mcid], "s+")
  qassert(midfield_term[, mcid], "s+")
  qassert(midfield_term[, institution], "s+")
  qassert(midfield_term[, term], "s+")

  # bind names due to NSE notes in R CMD check
  N <- NULL

  # do the work
  # Inner join using three columns of term
  x <- midfield_term[, .(mcid, institution, term)]
  y <- unique(dframe[, .(mcid)])
  DT <- y[x, on = .(mcid), nomatch = NULL]

  # count terms at institutions
  DT <- DT[, .N, by = c("mcid", "institution")]

  # what if there is a tie? can we select the most recent institution?
  # keep the institution with the most terms (if more than one)
  setkeyv(DT, c("mcid", "N"))
  DT <- DT[, .SD[.N], by = "mcid"]
  DT[, N := NULL]

  # join to dframe, overwrite institution if any
  if ("institution" %chin% names(dframe)) {
    dframe[, institution := NULL]
  }

  # left join, keep all rows of dframe
  setkeyv(DT, "mcid")
  setkeyv(dframe, "mcid")
  dframe <- DT[dframe]
  return(dframe)
}

# Using toy data
prep_fye_mice(toy_student, toy_term)

# Other columns, if any, are dropped
colnames(toy_student)
colnames(prep_fye_mice(toy_student, toy_term))

# Optional argument permits multiple CIP codes for FYE
prep_fye_mice(midfield_student = toy_student, 
              midfield_term =toy_term, 
              fye_codes = c("140101", "140102"))

# Documentation described below using an inline R code chunk, e.g.,
# "`r param_dots`" or "`r return_select_required`", are documented in the
# R/roxygen.R file.


#' Select required midfieldr variables
#'
#' Subset a data frame, selecting columns by matching or partially matching a
#' vector of character strings. A convenience function to reduce the dimensions
#' of a MIDFIELD data table at the start of a session by selecting only those
#' columns required by other midfieldr functions or that are required to form a
#' composite key. Particularly useful in interactive sessions when viewing the
#' data tables at various stages of an analysis.
#'
#' Several midfieldr functions require one or more of the variables `mcid`,
#' `institution`, `race`, `sex`, `^term`, `cip6`, and `level`. And if
#' one requires a composite key to uniquely identify rows in the course or
#' degree tables, the variables `abbrev`, `number` and degree variable
#' `degree` are also required. A vector of these names comprises the default
#' subset.
#'
#' Additional column names or partial names can be included by using the
#' `select_add` argument.
#'
#' The column names of `midfield_x` are searched for matches or partial matches
#' using `grep()`, thus search terms can include regular expressions. Variables
#' with names that match or partially match the search terms are returned; all
#' other columns are dropped. Rows are unaffected. Search terms not present are
#' silently ignored.
#'
#' One could use this function to select columns from a non-MIDFIELD data frame,
#' but with no benefit to the user---conventional column selection syntax is
#' better suited to that task. Here, we specialize the column selection to serve
#' midfieldr functions.
#'
#' @param midfield_x Data frame from which columns are selected.
#' @param ... `r param_dots`
#' @param select_add Character vector of additional column names to return.
#' @return `r return_select_required`
#'
#' @family select_*
#' @example man/examples/select_required_exa.R
#' @export
#'
select_required <- function(midfield_x, ..., select_add = NULL) {
  # remove all keys
  on.exit(setkey(midfield_x, NULL))

  # required argument
  qassert(midfield_x, "d+")

  # assert arguments after dots used by name
  wrapr::stop_if_dot_args(
    substitute(list(...)),
    paste(
      "Arguments after ... must be named.\n",
      "* Did you forget to write `select_add = `?\n *"
    )
  )

  # optional arguments
  default_select <- c(
    "mcid", "institution", "race", "sex", "^term", "cip6",
    "level", "abbrev", "number", "degree"
  )
  select <- c(default_select, select_add)
  select <- unique(select)

  # assertions for optional arguments
  qassert(select, "s+") # missing is OK

  # input modified (or not) by reference
  setDT(midfield_x)

  # required columns
  # NA
  # class of required columns
  # NA
  # bind names due to NSE notes in R CMD check
  # NA

  # do the work
  DT <- copy(midfield_x)
  on.exit(setkey(DT, NULL))

  # Create one string separated by OR
  search_pattern <- paste(select, collapse = "|")

  # Find names of columns matching or partially matching
  cols_we_want <- grep(search_pattern,
    names(DT),
    ignore.case = TRUE,
    value = TRUE
  )

  # Select with conventional data.table syntax
  DT <- DT[, .SD, .SDcols = cols_we_want]

  # stop if all columns have been eliminated
  if (length(names(DT)) < .Machine$double.eps^0.5) {
    stop(
      paste(
        "The result is empty. Possible causes are:\n",
        "* Column names of the input data frame\n",
        "  do not match any of the search terms.\n"
      ),
      call. = FALSE
    )
  }

  # enable printing (see data.table FAQ 2.23)
  DT[]
}

test_select_required <- function() {

    # usage
    # select_required(midfield_x, select_add = NULL)

    # Needed for tinytest::build_install_test()
    library("data.table")

    
    # Default character vector for selecting columns
    default_cols<- c("mcid", "institution", "race", "sex", "^term", "cip6", "level", "abbrev", "number", "degree")
    
    # Create one string separated by OR
    search_pattern <- paste(default_cols, collapse = "|")
    
    # Expect same number of columns as TRUE in grepl()
    x <- select_required(toy_student) 
    y <- grepl(search_pattern, names(x))
    expect_equal(ncol(x), sum(y))
    
    # Expect same number of columns as TRUE in grepl()
    x <- select_required(toy_term) 
    y <- grepl(search_pattern, names(x))
    expect_equal(ncol(x), sum(y))
    
    # Expect same number of columns as TRUE in grepl()
    x <- select_required(toy_course) 
    y <- grepl(search_pattern, names(x))
    expect_equal(ncol(x), sum(y))
    
    # Expect same number of columns as TRUE in grepl()
    x <- select_required(toy_degree) 
    y <- grepl(search_pattern, names(x))
    expect_equal(ncol(x), sum(y))
    
    # Add columns
    cols_to_add <- c("grade")
    x <- select_required(toy_course, select_add = cols_to_add) 
    y <- grepl(search_pattern, names(x))
    expect_equal(ncol(x), sum(y) + length(cols_to_add))
    
    # Silently ignore search terms not found
    x <- select_required(toy_student, select_add = cols_to_add) 
    y <- grepl(search_pattern, names(x))
    expect_equal(ncol(x), sum(y))

    # Error if no columns found
    cols_to_add <- c("grade")
    x <- toy_course[, .SD, .SDcols = cols_to_add]
    expect_error(select_required(x))

    # function output not printed
    invisible(NULL)
}

test_select_required()

# Start with an excerpt from the student data set 
dframe <- toy_student[1:10, .(mcid)]

# Add timely completion term column
add_timely_term(dframe, midfield_term = toy_term)

# Define timely completion as 200% of scheduled span (8 years)
add_timely_term(dframe, midfield_term = toy_term, span = 8)

# Existing timely_term column, if any, is overwritten
dframe[, timely_term := NA_character_]
add_timely_term(dframe, midfield_term = toy_term)

# Documentation described below using an inline R code chunk, e.g.,
# "`r param_dots`" or "`r return_filter_cip`", are
# documented in the R/roxygen.R file.


#' Subset rows that include matches to search strings
#'
#' Subset a CIP data frame, retaining rows that match or partially match a
#' vector of character strings. Columns are not subset unless selected in an
#' optional argument.
#'
#' Search terms can include regular expressions. Uses `grepl()`, therefore
#' non-character columns (if any) that can be coerced to character are also
#' searched for matches. Columns are subset by the values in `select` after the
#' search concludes.
#'
#' If none of the optional arguments are specified, the function returns the
#' original data frame.
#'
#' @param keep_text Character vector of search text for retaining rows,
#'        not case-sensitive. Can be empty if `drop_text` is used.
#' @param ... `r param_dots`
#' @param drop_text Optional character vector of search text for dropping
#'        rows, default NULL.
#' @param cip Data frame to be searched. Default `cip`.
#' @param select Optional character vector of column names to return,
#'        default all columns.
#' @return `r return_filter_cip`
#'
#' @family filter_*
#' @example man/examples/filter_cip_exa.R
#' @export
#'
filter_cip <- function(keep_text = NULL,
                       ...,
                       drop_text = NULL,
                       cip = NULL,
                       select = NULL) {
  # assert arguments after dots used by name
  wrapr::stop_if_dot_args(
    substitute(list(...)),
    paste(
      "Arguments after ... must be named.\n",
      "* Did you forget to write `drop_text = ` or `select = `?\n *"
    )
  )

  # optional arguments
  # cipx   <- cip %?% midfieldr::cip
  if (is.null(cip)) cip <- midfieldr::cip
  dframe <- copy(cip)
  select <- select %?% names(dframe)

  # remove all keys
  on.exit(setkey(dframe, NULL))

  # required argument
  qassert(dframe, "d+")

  # return if no work is being done
  if (identical(select, names(dframe)) &
    is.null(keep_text) &
    is.null(drop_text)) {
    setkey(dframe, NULL)
    return(dframe)
  }

  # assertions for optional arguments
  qassert(select, "s+") # missing is OK
  if (!is.null(keep_text)) qassert(keep_text, "s+")
  if (!is.null(drop_text)) qassert(drop_text, "s+")

  # input modified (or not) by reference
  setDT(dframe)

  # required columns
  # NA
  # class of required columns
  # NA
  # bind names due to NSE notes in R CMD check
  cip <- NULL

  # do the work
  dframe <- filter_char_frame(
    dframe = dframe,
    keep_text = keep_text,
    drop_text = drop_text
  )

  # stop if all rows have been eliminated
  if (abs(nrow(dframe) - 0) < .Machine$double.eps^0.5) {
    stop(
      paste(
        "The search result is empty. Possible causes are:\n",
        "* 'cip' contained no matches to terms in 'keep_text'.\n",
        "* 'drop_text' eliminated all remaining rows."
      ),
      call. = FALSE
    )
  }

  # message if a search term was not found
  # data frame with as many columns as there are keep_text terms
  # as many rows as there are being searched in data
  df <- data.frame(matrix("", nrow = nrow(dframe), ncol = length(keep_text)))
  names(df) <- keep_text

  for (j in seq_along(keep_text)) {
    df[, j] <- apply(dframe, 1, function(i) {
      any(grepl(keep_text[j], i, ignore.case = TRUE))
    })
  }

  # the sum is 0 for all FALSE in a column for that search term
  sumTF <- colSums(df)
  not_found <- sumTF[!sapply(sumTF, as.logical)]
  if (length(not_found) > 0) {
    message(paste(
      "Can't find these terms:",
      paste(names(not_found), collapse = ", ")
    ))
  }

  # subset columns
  dframe <- dframe[, .SD, .SDcols = select]

  # enable printing (see data.table FAQ 2.23)
  dframe[]
}

# ------------------------------------------------------------------------

# Subset rows of character data frame by matching keep_texts

# dframe      data frame of character variables
# keep_text   character vector of search keep_texts for retaining rows
# drop_text   character vector of search keep_texts for dropping rows

filter_char_frame <- function(dframe,
                              keep_text = NULL,
                              drop_text = NULL) {
  DT <- as.data.table(dframe)

  # filter to keep rows
  if (length(keep_text) > 0) {
    keep_text <- paste0(keep_text, collapse = "|")
    DT <- DT[apply(DT, 1, function(i) {
      any(grepl(keep_text, i, ignore.case = TRUE))
    }), ]
  }

  # filter to drop rows
  if (length(drop_text) > 0) {
    drop_text <- paste0(drop_text, collapse = "|")
    DT <- DT[apply(DT, 1, function(j) {
      !any(grepl(drop_text, j, ignore.case = TRUE))
    }), ]
  }

  # works by reference
  return(DT)
}

test_filter_cip <- function() {

    # usage
    # filter_cip(keep_text = NULL,
    #            ...,
    #            drop_text = NULL,
    #            cip = NULL, 
    #            select = NULL)
    
    # uncomment for internal manual testing
    # library(tinytest)
    
    # Needed for tinytest::build_install_test()
    library("data.table")

    # test case
    music_cip <- wrapr::build_frame(
          "cip4"  , "cip4name", "cip6"  , "cip6name"               |
            "5009", "Music"   , "500901", "Music, General"         |
            "5009", "Music"   , "500906", "Conducting"             |
            "5009", "Music"   , "500907", "Piano and Organ"        |
            "5009", "Music"   , "500908", "Voice and Opera"        |
            "5009", "Music"   , "500910", "Jazz, Jazz Studies"     |
            "5009", "Music"   , "500912", "Music Pedagogy"         |
            "5009", "Music"   , "500913", "Music Technology"       |
            "5009", "Music"   , "500914", "Brass Instruments"      |
            "5009", "Music"   , "500915", "Woodwind Instruments"   |
            "5009", "Music"   , "500916", "Percussion Instruments" |
            "5009", "Music"   , "500999", "Music, Other")
    setDT(music_cip)

    # various answers
    select_these <- c("cip4", "cip4name")
    music_codes <- music_cip[, cip6]
    ans <- wrapr::build_frame(
        "cip4", "cip4name" |
            "5009",  "Music"
    )
    setDT(ans)

    # Result is correct
    expect_equal(unique(filter_cip(cip = music_cip, select = select_these)), ans)

    # Arguments before ... do not have to be named if ordered correctly
    expect_equal(
        filter_cip(keep_text = c("500913"), cip = music_cip),
        filter_cip(c("500913"), cip = music_cip)
    )

    # Correct result with keep and select
    expect_equal(
        filter_cip(keep_text = "Conducting", cip = music_cip, select = "cip6"),
        music_cip[cip6name =="Conducting", .(cip6)]
    )

    # Data returned unaltered if no arguments do anything
    expect_equal(filter_cip(cip = music_cip), setkey(music_cip, NULL))

    # Select alone produces correct columns
    expect_equal(colnames(filter_cip(cip = music_cip, select = select_these)),
                 select_these)

    # Result is empty if drop_text is all-inclusive
    expect_error(
        filter_cip(cip = music_cip, drop_text = music_codes),
    )

    # Result is empty if keep_text is misspelled and therefore not found
    expect_error(
        filter_cip(keep_text = "Brasss", cip = music_cip)
    )

    # Message if some keep_text not found
    incorrect_cip_text <- c("Bogus", "111111")
    expect_message(
        filter_cip(keep_text = c(music_codes, incorrect_cip_text), cip = music_cip)
    )

    # Non-character columns coerced to text
    DT <- copy(music_cip[, .(cip4name, cip6)])
    DT[, x_dbl  := as.numeric(cip6)/100]
    DT[, x_int  := .I]
    DT[, x_lgcl := TRUE]

    # expect_equal(filter_cip("Music", cip = DT))
    expect_equal(filter_cip("11", cip = DT), DT[x_int == 11L])
    expect_equivalent(filter_cip("TRUE", cip = DT), DT)

    # Default `cip`
    expect_equal(
        filter_cip("500913"),
        filter_cip(keep_text = c("500913"), cip = cip)
    )
    
    # function output not printed
    invisible(NULL)
}

test_filter_cip()

# Documentation described below using an inline R code chunk, e.g.,
# "`r dframe_add_completion_status`" or "`r return_data_frame`", are documented
# in the R/roxygen.R file.


#' Determine completion status for every student
#'
#' Add columns to a data frame of student-level records that indicate whether a
#' student completed a degree, and if so, whether their completion was timely.
#'
#' By "completion" we mean an undergraduate earning their first baccalaureate
#' degree (or degrees, for students earning more than one degree in the same
#' term). Additional degrees, if any, earned later than the term of the first
#' degree are ignored.
#'
#' In many studies, students must complete a degree in a specified time span,
#' for example 4-, 6-, or 8-years after admission. If they do, their completion
#' is timely; if not, their completion is late and they are grouped with the
#' non-completers when computing a metric such as graduation rate.
#'
#' Completion status is "timely" for students completing a degree no later than
#' their timely completion terms. See also `add_timely_term()`.
#'
#' @param dframe          `r dframe_add_completion_status`
#' @param midfield_degree `r midfield_degree_add_completion_status`
#'
#' @return `r return_data_frame`
#' \describe{
#'  \item{`term_degree`}{Character. Term in which the first degree(s) are
#'  completed. Encoded YYYYT. Joined from `midfield_degree` data table.}
#'  \item{`completion_status`}{Character. Label each observation to
#'  indicate completion status. Possible values are: "timely", indicating
#'  completion no later than the timely completion term; "late", indicating
#'  completion after the timely completion term; and "NA" indicating
#'  non-completion.}
#' }
#'
#' @family add_*
#' @example man/examples/add_completion_status_exa.R
#' @export
#'
add_completion_status <- function(dframe, midfield_degree = degree) {
  on.exit(setkey(dframe, NULL), add = TRUE)
  on.exit(setkey(midfield_degree, NULL), add = TRUE)

  # required arguments
  qassert(dframe, "d+")
  qassert(midfield_degree, "d+")

  # inputs modified (or not) by reference
  setDT(dframe)
  setDT(midfield_degree) # immediately subset, so side-effect OK

  # required columns
  assert_names(colnames(dframe),
    must.include = c("mcid", "timely_term")
  )
  assert_names(colnames(midfield_degree),
    must.include = c("mcid", "term_degree")
  )

  # class of required columns
  qassert(dframe[, mcid], "s+")
  qassert(dframe[, timely_term], "s+")
  qassert(midfield_degree[, mcid], "s+")
  qassert(midfield_degree[, term_degree], "s+")

  # bind names due to NSE notes in R CMD check
  completion_status <- NULL
  timely_term <- NULL
  term_degree <- NULL

  # do the work

  # variables added by this function and functions called (if any)
  new_cols <- c("term_degree", "completion_status")

  # retain original variables NOT in the vector of new columns
  old_cols <- find_old_cols(dframe, new_cols)
  dframe <- dframe[, .SD, .SDcols = old_cols]

  # Inner join using three columns of term
  x <- midfield_degree[, .(mcid, term_degree)]
  y <- unique(dframe[, .(mcid)])
  DT <- y[x, on = .(mcid), nomatch = NULL]

  # keep the first degree term
  setorderv(DT, c("mcid", "term_degree"))
  DT <- na.omit(DT, cols = c("term_degree"))
  DT <- DT[, .SD[1], by = "mcid"]
  setkey(DT, NULL)

  # left-outer join, keep all rows of dframe
  dframe <- merge(dframe, DT, by = "mcid", all.x = TRUE)

  # completion is timely, late, or NA
  dframe[, completion_status := fifelse(term_degree <= timely_term,
    "timely",
    "late",
    na = NA_character_
  )]

  # select columns to return
  final_cols <- c(old_cols, new_cols)
  dframe <- dframe[, .SD, .SDcols = final_cols]

  # old columns as keys, order columns and rows
  set_colrow_order(dframe, old_cols)

  # enable printing (see data.table FAQ 2.23)
  dframe[]
}

test_add_completion_status <- function() {

    # usage
    # add_completion_status(dframe, midfield_degree = degree)

    # Needed for tinytest::build_install_test()
    library("data.table")

    # create answers
    dframe <- toy_student[1:10, .(mcid)]
    dframe <- add_timely_term(dframe, midfield_term = toy_term)
    
    # DT <- add_completion_status(dframe, midfield_degree = toy_degree)
    # cat(wrapr::draw_frame(DT))

    DT <- wrapr::build_frame(
        "mcid"         , "term_i", "level_i"     , "adj_span", "timely_term", "term_degree"  , "completion_status" |
            "MID25784187", "19885" , "01 Freshman" , 6         , "19943"      , "19946"      , "late"              |
            "MID25784974", "19883" , "02 Sophomore", 5         , "19931"      , NA_character_, NA_character_       |
            "MID25816209", "19881" , "02 Sophomore", 5         , "19923"      , NA_character_, NA_character_       |
            "MID25819358", "19946" , "02 Sophomore", 5         , "19993"      , "19963"      , "timely"            |
            "MID25828870", "19881" , "01 Freshman" , 6         , "19933"      , "19923"      , "timely"            |
            "MID25829749", "19995" , "03 Junior"   , 4         , "20033"      , NA_character_, NA_character_       |
            "MID25841418", "19981" , "03 Junior"   , 4         , "20013"      , "19993"      , "timely"            |
            "MID25845197", "19905" , "03 Junior"   , 4         , "19943"      , "19921"      , "timely"            |
            "MID25846316", "19911" , "01 Freshman" , 6         , "19963"      , "19951"      , "timely"            |
            "MID25847220", "19891" , "01 Freshman" , 6         , "19943"      , "19933"      , "timely"            )
    setDT(DT)

    # correct answers
    expect_equal(
        DT,
        add_completion_status(dframe, toy_degree)
    )

    # overwrites existing columns
    expect_equal(
        DT,
        add_completion_status(DT, toy_degree)
    )

    # midfield_degree argument must be degree or equivalent
    expect_error(
        add_completion_status(DT, midfield_degree = toy_student)
    )

    # dframe argument requires ID and timely_term columns
    col_missing <- copy(DT)[, mcid := NULL]
    expect_error(
        add_completion_status(col_missing, midfield_degree = toy_degree)
    )
    col_missing <- copy(DT)[, timely_term := NULL]
    expect_error(
        add_completion_status(col_missing, midfield_degree = toy_degree)
    )

    # midfield_degree argument requires ID and term columns
    col_missing <- copy(toy_degree)[, mcid := NULL]
    expect_error(
        add_completion_status(DT, midfield_degree = col_missing)
    )
    col_missing <- copy(toy_degree)[, term_degree := NULL]
    expect_error(
        add_completion_status(DT, midfield_degree = col_missing)
    )

    # data frame arguments must be data frames
    expect_error(
        add_completion_status(DT$mcid, toy_degree)
    )
    expect_error(
        add_completion_status(DT, toy_degree$term)
    )

    # required arguments must be explicit
    expect_error(
        add_completion_status(NULL, toy_degree)
    )
    expect_error(
        add_completion_status(DT, NULL)
    )

    invisible(NULL)
}

test_add_completion_status()

# Documentation described below using an inline R code chunk, e.g.,
# "`r dframe_add_timely_term`" or "`r return_data_frame`", are
# documented in the R/roxygen.R file.


#' Calculate a timely completion term for every student
#'
#' Add a column to a data frame of student-level records that indicates the
#' latest term by which degree completion  would be considered timely for every
#' student.
#'
#' By "completion" we mean an undergraduate earning their first baccalaureate
#' degree (or degrees, for students earning more than one degree in the same
#' term).
#'
#' In many studies, students must complete their programs in a specified time
#' span, for example 4-, 6-, or 8-years after admission. If they do, their
#' completion is timely; if not, their completion is late and they are grouped
#' with the non-completers when computing a metric such as graduation rate.
#'
#' Our heuristic assigns `span` number of years (default is 6 years) to every
#' student. For students admitted at second-year level or higher, the span is
#' reduced by one year for each full year the student is assumed to have
#' completed. For example, a student admitted at the second-year level is
#' assumed to have completed one year of a program, so their span is reduced by
#' one year.
#'
#' The adjusted span is added to the initial term to create the timely
#' completion term in the `timely_term` column.
#'
#' @param dframe        `r dframe_add_timely_term`
#' @param midfield_term `r midfield_term_add_timely_term`
#' @param ...           `r param_dots`
#' @param span Optional integer scalar, number of years to define timely
#'   completion. Commonly used values are are 100, 150, and 200 percent of
#'   `sched_span.` Default 6 years.
#' @param sched_span Optional integer scalar, the number of years an institution
#'   officially schedules for completing a program. Default 4 years.
#'
#' @return `r return_data_frame`
#' \describe{
#'  \item{`term_i`}{Character. Initial term of a student's longitudinal
#'  record, encoded YYYYT}
#'  \item{`level_i`}{Character. Student level (01 Freshman, 02 Sophomore,
#'  etc.) in their initial term}
#'  \item{`adj_span`}{Numeric. Integer span of years for timely completion
#'  adjusted for a student's initial level.}
#'  \item{`timely_term`}{Character. Latest term by which program
#'  completion would be considered timely for every student. Encoded YYYYT.}
#' }
#'
#' @family add_*
#' @example man/examples/add_timely_term_exa.R
#' @export
#'
add_timely_term <- function(dframe,
                            midfield_term = term,
                            ...,
                            span = NULL,
                            sched_span = NULL) {
  # remove keys if any
  on.exit(setkey(dframe, NULL))
  on.exit(setkey(midfield_term, NULL), add = TRUE)

  # required arguments
  qassert(dframe, "d+")
  qassert(midfield_term, "d+")

  # assert arguments after dots used by name
  wrapr::stop_if_dot_args(
    substitute(list(...)),
    paste(
      "Arguments after ... must be named.\n",
      "* Did you forget to write `details = ` or `span = `?\n *"
    )
  )

  # optional arguments
  span <- span %?% 6
  sched_span <- sched_span %?% 4

  # optional arguments assertions
  assert_int(sched_span, lower = 0)
  assert_int(span, lower = sched_span)

  # ensure data.table format
  setDT(dframe)
  setDT(midfield_term) # immediately subset, so no changes by reference

  # required columns
  assert_names(colnames(dframe),
    must.include = c("mcid")
  )
  assert_names(colnames(midfield_term),
    must.include = c("mcid", "term", "level")
  )

  # class of required columns
  qassert(dframe[, mcid], "s+")
  qassert(midfield_term[, mcid], "s+")
  qassert(midfield_term[, term], "s+")
  qassert(midfield_term[, level], "s+")

  # bind names due to NSE notes in R CMD check
  timely_term <- NULL
  adj_span <- NULL
  level_i <- NULL
  term_i <- NULL
  delta <- NULL
  yyyy <- NULL

  # do the work

  # variables added by this function and functions called (if any)
  initial_traits_cols <- c("term_i", "level_i")
  new_cols <- c(initial_traits_cols, "adj_span", "timely_term")

  # retain original variables NOT in the vector of new columns
  old_cols <- find_old_cols(dframe, new_cols)
  dframe <- dframe[, .SD, .SDcols = old_cols]

  # begin
  DT <- copy(dframe)

  # add first recorded term_i and level_i
  DT <- add_initial_term(DT, midfield_term)
  DT <- add_initial_level(DT, midfield_term)
  DT <- DT[, .SD, .SDcols = c("mcid", initial_traits_cols)]

  # begin constructing the timely term
  DT[, `:=`(
    yyyy = substr(term_i, 1, 4),
    t    = substr(term_i, 5, 5)
  )]

  # for month terms, (letters A, B, C, ...), set first term to zero
  DT <- DT[t %chin% LETTERS | t %chin% letters, t := "0"]

  # make year and term numeric
  DT[, `:=`(
    yyyy = as.numeric(yyyy),
    t    = as.numeric(t)
  )]

  # if first term is in summer, delay to the subsequent Fall
  DT[t > 3, `:=`(yyyy = yyyy + 1, t = 1)]

  # reduce span by assumed number of completed years by level
  DT[, delta := fcase(
    level_i %like% "04", 3,
    level_i %like% "03", 2,
    level_i %like% "02", 1,
    default = 0
  )]
  DT[, adj_span := span - delta]

  # use adj_span to construct estimated timely-completion term
  DT[t == 0 | t == 1, timely_term := paste0(yyyy + adj_span - 1, 3)]
  DT[t > 1, timely_term := paste0(yyyy + adj_span, 1)]

  # remove all but essential variables
  DT <- DT[, .SD, .SDcols = c("mcid", new_cols)]

  # ensure no duplicate rows
  setkeyv(DT, "mcid")
  DT <- DT[, .SD[1], by = "mcid"]

  # left join new columns to dframe by key(s)
  setkeyv(dframe, "mcid")
  dframe <- DT[dframe]

  # select columns to return
  final_cols <- c(old_cols, new_cols)
  dframe <- dframe[, .SD, .SDcols = final_cols]

  # old columns as keys, order columns and rows
  set_colrow_order(dframe, old_cols)

  # enable printing (see data.table FAQ 2.23)
  dframe[]
}

# ------------------------------------------------------------------------

# Add students' initial terms

add_initial_term <- function(dframe, midfield_term) {
  # remove keys if any
  on.exit(setkey(dframe, NULL))
  on.exit(setkey(midfield_term, NULL), add = TRUE)

  # ensure data.table format, changes by reference
  setDT(dframe)
  setDT(midfield_term)

  # variables added by this function
  new_cols <- c("term_i")

  # retain original variables NOT in the vector of new columns
  old_cols <- find_old_cols(dframe, new_cols)
  dframe <- dframe[, .SD, .SDcols = old_cols]

  # obtain new_cols keyed by ID
  # DT <- filter_match(
  #     dframe = midfield_term,
  #     match_to = dframe,
  #     by_col = "mcid",
  #     select = c("mcid", "term")
  # )


  # Inner join using two columns of term
  x <- midfield_term[, .(mcid, term)]
  y <- unique(dframe[, .(mcid)])
  DT <- y[x, on = .(mcid), nomatch = NULL]


  # retain first term by ID
  setkeyv(DT, c("mcid", "term"))
  DT <- DT[, .SD[1], by = c("mcid")]

  # rename new columns
  setnames(DT,
    old = c("term"),
    new = c("term_i")
  )

  # left join new columns to dframe by key(s)
  setkeyv(DT, "mcid")
  setkeyv(dframe, "mcid")
  dframe <- DT[dframe]

  # enable printing (see data.table FAQ 2.23)
  dframe[]
}

# ------------------------------------------------------------------------

# Add students' initial levels

add_initial_level <- function(dframe, midfield_term) {
  # remove keys if any
  on.exit(setkey(dframe, NULL))
  on.exit(setkey(midfield_term, NULL), add = TRUE)

  # ensure data.table format, changes by reference
  setDT(dframe)
  setDT(midfield_term)

  # variables added by this function and functions called (if any)
  new_cols <- c("level_i")

  # retain original variables NOT in the vector of new columns
  old_cols <- find_old_cols(dframe, new_cols)
  dframe <- dframe[, .SD, .SDcols = old_cols]

  # # obtain new_cols keyed by ID
  # DT <- filter_match(
  #     dframe = midfield_term,
  #     match_to = dframe,
  #     by_col = "mcid",
  #     select = c("mcid", "term", "level")
  # )

  # Inner join using three columns of term
  x <- midfield_term[, .(mcid, term, level)]
  y <- unique(dframe[, .(mcid)])
  DT <- y[x, on = .(mcid), nomatch = NULL]

  # retain first term by ID
  setkeyv(DT, c("mcid", "term"))
  DT <- DT[, .SD[1], by = c("mcid")]

  # rename new columns
  setnames(DT,
    old = c("level"),
    new = c("level_i")
  )

  # left join new columns to dframe by key(s)
  setkeyv(DT, "mcid")
  setkeyv(dframe, "mcid")
  dframe <- DT[dframe]

  # enable printing (see data.table FAQ 2.23)
  dframe[]
}

test_add_timely_term <- function() {

    # usage
    # add_timely_term(
    #     dframe,
    #     midfield_term = term,
    #     ...,
    #     span = NULL,
    #     sched_span = NULL
    # )

    # Needed for tinytest::build_install_test()
    library("data.table")

    # create case
    # library("midfieldr")
    # x <- toy_student[1:10, .(mcid)]
    # y <- add_timely_term(x, toy_term)
    # DT <- cat(wrapr::draw_frame(y))
    DT <- wrapr::build_frame(
        "mcid"         , "term_i", "level_i"     , "adj_span", "timely_term" |
            "MID25784187", "19885" , "01 Freshman" , 6         , "19943"       |
            "MID25784974", "19883" , "02 Sophomore", 5         , "19931"       |
            "MID25816209", "19881" , "02 Sophomore", 5         , "19923"       |
            "MID25819358", "19946" , "02 Sophomore", 5         , "19993"       |
            "MID25828870", "19881" , "01 Freshman" , 6         , "19933"       |
            "MID25829749", "19995" , "03 Junior"   , 4         , "20033"       |
            "MID25841418", "19981" , "03 Junior"   , 4         , "20013"       |
            "MID25845197", "19905" , "03 Junior"   , 4         , "19943"       |
            "MID25846316", "19911" , "01 Freshman" , 6         , "19963"       |
            "MID25847220", "19891" , "01 Freshman" , 6         , "19943"       )
    setDT(DT)
   
    # correct answers, add optional arguments in combinations
    x <- toy_student[1:10, .(mcid)]
    # without detail
    expect_equal(
        DT,
        add_timely_term(x, toy_term)
    )
    expect_equal(
        DT,
        add_timely_term(x, toy_term, span = 6)
    )

    # arguments must be data frames
    expect_error(add_timely_term(toy_student$mcid, toy_term))
    expect_error(add_timely_term(toy_student[, mcid], toy_term$term))

    # arguments after ... must be named
    expect_error(add_timely_term(x,
                                 toy_term,
                                 detail = FALSE,
                                 6)) # span not named

    # optional arguments are specific types
    expect_error(
        add_timely_term(x, toy_term, span = TRUE)
    )

    # optional arguments, NA same as NULL
    expect_equal(
        add_timely_term(x, toy_term, span = 6),
        add_timely_term(x, toy_term, span = NA)
    )

    # span must be >=  sched_span
    expect_error(
        add_timely_term(x, toy_term, span = 2)
    )
    expect_error(
        add_timely_term(x, toy_term, span = 0, sched_span = 1)
    )

    invisible(NULL)
}

test_add_timely_term()

# Documentation described below using an inline R code chunk, e.g.,
# "`r var_mcid`" or "`r var_institution`", are documented in the
# R/roxygen.R file.


# -------------------------------------------------------------------

#' Table of academic programs
#'
#' A data table based on the US National Center for Education Statistics (NCES),
#' Integrated Postsecondary Education Data System (IPEDS), 2010 CIP,
#' \url{http://nces.ed.gov/ipeds/cipcode/}. The data are codes and names for
#' 1582 instructional programs organized on three levels: a 2-digit series, a
#' 4-digit series, and a 6-digit series.
#'
#' The midfielddata taxonomy includes one non-IPEDS code (999999) for Undecided
#' or Unspecified, instances in which institutions reported no program
#' information or that students were not enrolled in a program.
#'
#' The MIDFIELD research database include CIPs for undergraduate pre-majors
#' such as pre-med (511102), pre-law (220001), and pre-vet (511104).
#'
#' @format `data.table` with 1582 rows and 6 columns keyed by the 6-digit CIP
#'   code:
#'
#' \describe{
#'
#'   \item{`cip6`}{Character 6-digit code representing "specific
#'   instructional programs" (US National Center for Education Statistics).}
#'
#'   \item{`cip6name`}{Character program name at the 6-digit level}
#'
#'   \item{`cip4`}{Character 4-digit code (the first 4 digits of `cip6`)
#'   representing "intermediate groupings of programs that have
#'   comparable content and objectives."}
#'
#'   \item{`cip4name`}{Character program name at the 4-digit level.}
#'
#'   \item{`cip2`}{Character 2-digit code (the first 2 digits of `cip6`)
#'   representing "the most general groupings of related programs."}
#'
#'   \item{`cip2name`}{Character program name at the 2-digit level.}
#'
#' }
#'
#' @family cip-data
#'
"cip"

# -------------------------------------------------------------------

#' Starting program proxies for FYE students
#'
#' Proxies are the degree-granting engineering programs we estimate that
#' First-Year Engineering (FYE) students would have declared had they not been
#' required to enroll in FYE. Keyed by student ID. Proxies are provided for all
#' students in the midfielddata practice data who enroll in FYE in their first
#' term.
#'
#' The proxy variable contains 6-digit CIP codes of degree-granting engineering
#' programs, e.g., Electrical Engineering, Mechanical Engineering, etc., that
#' are substituted for the FYE CIP code when an analysis requires
#' degree-granting starting programs. The most common application is a
#' graduation rate calculation.
#'
#' The estimation is based on students' first post-FYE programs and a multiple
#' imputation suitable for categorical variables using the mice package. The
#' predictor variables are institution, race, and sex. The estimated variable is
#' the 6-digit CIP code of a degree-granting engineering program at their
#' institution.
#'
#' `fye_proxy` holds only for the practice data in midfielddata---these values
#' cannot be commingled with the MIDFIELD research database.
#'
#' @format `data.table` with 4623 rows and 2 columns keyed by student ID:
#' \describe{
#'   `r var_mcid`
#'   `r var_proxy`
#'  }
#'
#' @family cip-data
#'
"fye_proxy"

# -------------------------------------------------------------------

#' Student data for examples
#'
#' Selected variables modeled on those in the `student` practice data for use in
#' package examples and articles. Sampled from an early version of the practice
#' data, the toy data are not a current practice data sample.
#'
#' @format `data.table` with 99 rows and 4 columns keyed by student ID:
#' \describe{
#'  `r var_mcid`
#'  `r var_institution`
#'  `r var_race`
#'  `r var_sex`
#' }
#'
#' @family toy-data
#'
"toy_student"

# -------------------------------------------------------------------

#' Course data for examples
#'
#' Selected variables modeled on those in the `course` practice data for use in
#' package examples and articles. Sampled from an early version of the practice
#' data, the toy data are not a current practice data sample.
#'
#' @format `data.table` with 4616 rows and 6 columns keyed by student ID:
#' \describe{
#'  `r var_mcid`
#'  `r var_institution`
#'  `r var_term`
#'  `r var_abbrev`
#'  `r var_number`
#'  `r var_grade`
#' }
#'
#' @family toy-data
#'
"toy_course"

# -------------------------------------------------------------------

#' Term data for examples
#'
#' Selected variables modeled on those in the `term` practice data for use in
#' package examples and articles. Sampled from an early version of the practice
#' data, the toy data are not a current practice data sample.
#'
#' @format `data.table` with 150 rows and 5 columns keyed by student ID.
#' The variables are:
#' \describe{
#'  `r var_mcid`
#'  `r var_institution`
#'  `r var_term`
#'  `r var_cip6`
#'  `r var_level`
#' }
#'
#' @family toy-data
#'
"toy_term"

# -------------------------------------------------------------------

#' Degree data for examples
#'
#' Selected variables modeled on those in the `degree` practice data for use in
#' package examples and articles. Sampled from an early version of the practice
#' data, the toy data are not a current practice data sample.
#'
#' @format `data.table` with 65 rows and 4 columns keyed by student ID.
#' The variables are:
#' \describe{
#'  `r var_mcid`
#'  `r var_institution`
#'  `r var_term_degree`
#'  `r var_cip6`
#' }
#'
#' @family toy-data
#'
"toy_degree"

# -------------------------------------------------------------------

#' Case-study program labels and codes
#'
#' Data table of program CIP codes and labels of the four programs of the case
#' study. Keyed by 6-digit CIPs. Provided for the convenience of vignette users.
#'
#' Starting with the midfieldr `cip` data set, we extracted the CIPs of the four
#' programs of the case study and assigned them a custom label to be used for
#' grouping and summarizing.
#'
#' @format `data.table` with 15 rows and 2 columns.
#' The variables are:
#' \describe{
#'  `r var_cip6`
#'  \item{`program`}{Character, abbreviated labels for four engineering
#'  programs. Values are "CE" (Civil Engineering), "EE" (Electrical
#'  Engineering), "ISE" (Industrial/Systems Engineering), and  "ME" (Mechanical
#'  Engineering).}
#' }
#'
#' @family case-study-data
#'
"study_programs"

# -------------------------------------------------------------------

#' Case-study observations
#'
#' Data table of post-processed observations of students ever enrolled in, and
#' students graduating from, the four programs of the case study. Keyed by
#' student ID. Provided for the convenience of vignette users.
#'
#' Starting with the case-study starting pool of students ever enrolled in the
#' four programs of the study (Civil, Electrical, Industrial/Systems, and
#' Mechanical Engineering), we filtered the data for data sufficiency, degree
#' seeking, program, and timely completion.
#'
#' A data frame of "ever enrolled" and a data frame of "timely graduates" were
#' bound using shared column names and are distinguished in the `bloc` variable.
#' This data structure facilitates grouping and summarizing by race, sex,
#' program, and group.
#'
#' @format `data.table` with 8917 rows and 5 columns.
#' The variables are:
#' \describe{
#'  `r var_mcid`
#'  `r var_race`
#'  `r var_sex`
#'  `r var_program`
#'  `r var_bloc`
#' }
#'
#' @family case-study-data
#'
"study_observations"

# -------------------------------------------------------------------

#' Case-study results
#'
#' Data table of longitudinal stickiness for the four programs of the case study
#' (Civil, Electrical, Industrial/Systems, and Mechanical Engineering) grouped
#' by program, race/ethnicity, and sex. Provided for the convenience of vignette
#' users.
#'
#' Longitudinal stickiness is the ratio of the number of students graduating
#' from a program to the number of students ever enrolled in the program over
#' the time span of available data. Results are based on data that have been
#' filtered for data sufficiency, degree seeking, and timely completion.
#'
#' @format `data.table` with 50 rows and 6 columns:
#' \describe{
#'  `r var_program`
#'  `r var_sex`
#'  `r var_race`
#'  `r var_ever_enrolled`
#'  `r var_graduates`
#'  `r var_stickiness`
#' }
#'
#' @family case-study-data
#'
"study_results"

# -------------------------------------------------------------------

#' Baseline ID bloc to start a typical analysis
#'
#' Data frame of IDs after processing the practice data for data sufficiency and
#' degree seeking. Provides a convenient bloc to start many of the analysis
#' illustrated in the package articles.
#'
#' @format `data.table` with 76875 rows and 1 column:
#' \describe{
#'   `r var_mcid`
#' }
#'
#' @family case-study-data
#'
"baseline_mcid"


# -------------------------------------------------------------------

#' Grade scale
#'
#' Data frame of letter grades and conventional point assignments used for
#' computing grade point averages.
#'
#' @format `data.table` with 12 rows and 2 columns:
#' \describe{
#'   \item{`letter_grade`}{Character, letter grades using the conventional US
#'       scale from A to F.}
#'   \item{`points`}{Numerical, 4.0 scale of points assigned to letter grades.}
#' }
#'
#' @family scales
#'
"grade_scale"

# -------------------------------------------------------------------

#' ACT-SAT conversion scale
#'
#' Data frame for converting between ACT and SAT scores. A range of SAT scores
#' converts to a single ACT score; an ACT score converts to a single
#' value equivalent SAT score.
#'
#' @format `data.table` with 28 rows and 4 columns:
#' \describe{
#'   \item{`act_comp`}{Numerical, ACT composite score.}
#'   \item{`sat_lower`}{Numerical, total SAT, lower limit of range
#'   corresponding to the ACT composite score.}
#'   \item{`sat_equiv`}{Numerical, total SAT, value to use when
#'   converting ACT score to a single SAT score.}
#'   \item{`sat_upper`}{Numerical, total SAT, upper limit of range
#'   corresponding to the ACT composite score.}
#' }
#'
#' @family scales
#' @source ACT/SAT Concordance (2018) ACT Education Corp. \url{https://www.act.org/content/dam/act/unsecured/documents/ACT-SAT-Concordance-Tables.pdf}
#'
"act_sat_scale"

# Documentation described below using an inline R code chunk, e.g.,
# "`r midfield_student_prep_fye_mice`" or "`r return_prep_fye_mice`", are
# documented in the R/roxygen.R file.


#' Prepare FYE data for multiple imputation
#'
#' Constructs a data frame of student-level records of First-Year Engineering
#' (FYE) programs and conditions the data for later use as an input to the mice
#' R package for multiple imputation. Sets up three variables as predictors
#' (institution, race/ethnicity, and sex) and one variable to be estimated
#' (program CIP code).
#'
#' At some US institutions, engineering students are required to complete a
#' First-Year Engineering (FYE) program as a prerequisite for declaring an
#' engineering major. Administratively, degree-granting engineering programs
#' such as Electrical Engineering or Mechanical Engineering treat their incoming
#' post-FYE students as their "starting" cohorts.  However, when computing a
#' metric that requires a count of starters---graduation rate, for example---FYE
#' records must be treated with special care to avoid a miscount.
#'
#' To illustrate the potential for miscounting starters, suppose we wish to
#' calculate a Mechanical Engineering (ME) graduation rate. Students starting in
#' ME constitute the starting pool and the fraction of that pool graduating in
#' ME is the graduation rate. At FYE institutions, an ME program would typically
#' define their starting pool as the post-FYE cohort entering their program.
#' This may be the best information available, but it invariably undercounts
#' starters by failing to account for FYE students who do not transition
#' (post-FYE) to degree-granting engineering programs---students who may have
#' left the institution or switched to non-engineering majors. In either case,
#' in the absence of the FYE requirement, some of these students would have been
#' ME starters. By neglecting these students, the count of ME starters is
#' artificially low resulting in an ME graduation rate that is artificially
#' high. The same is true for every degree-granting engineering discipline in an
#' FYE institution.
#'
#' Therefore, to avoid miscounting starters at FYE institutions, we have to
#' estimate an "FYE proxy", that is, the 6-digit CIP codes of the
#' degree-granting engineering programs that FYE students would have declared
#' had they not been required to enroll in FYE. The purpose of
#' `prep_fye_mice()`` is to prepare the data for making that estimation.
#'
#' After running `prep_fye_mice()` but before running `mice()`, one can edit
#' variables or add variables to create a custom set of predictors. The mice
#' package expects all predictors and the proxy variables to be factors. Do not
#' delete the institution variable because it ensures that a student's imputed
#' program is available at their institution.
#'
#' In addition, ensure that the only missing values are in the proxy column.
#' Other variables are expected to be complete (no NA values). A value of
#' "unknown" in a predictor column, e.g., race/ethnicity or sex, is an
#' acceptable value, not missing data. Observations with missing or unknown
#' values in the ID or institution columns (if any) should be removed.
#'
#' @param midfield_student `r midfield_student_prep_fye_mice`
#' @param midfield_term    `r midfield_term_prep_fye_mice`
#' @param ...              `r param_dots`
#' @param fye_codes Optional character vector of 6-digit CIP codes to identify
#'   FYE programs, default "140102". Codes must be 6-digit strings of numbers;
#'   regular expressions are prohibited. Non-engineering codes---those that do
#'   not start with 14---produce an error.
#'
#' @return `r return_prep_fye_mice`
#'  \describe{
#'    \item{`mcid`}{Character, anonymized student identifier. Returned
#'            as-is.}
#'    \item{`race`}{Factor, race/ethnicity as self-reported by the
#'            student. An imputation predictor variable.}
#'    \item{`sex`}{Factor, sex as self-reported by the
#'            student. An imputation predictor variable.}
#'    \item{`institution`}{Factor, anonymized institution name. An
#'            imputation predictor variable.}
#'    \item{`proxy`}{Factor, 6-digit CIP code of a student's known,
#'            post-FYE engineering program or NA representing missing
#'            values to be imputed.}
#'  }
#'
#' @section Method: The function extracts all terms for all FYE students,
#'   including those who migrate to enter Engineering after their first term,
#'   and identifies the first post-FYE program in which they enroll, if any.
#'   This treatment yields two possible outcomes for values returned in the
#'   `proxy` column:
#'
#' \enumerate{
#' \item{The student completes FYE and enrolls in an engineering major. For
#'     this outcome, we know that at the student's first opportunity, they
#'     enrolled in an engineering program of their choosing. The CIP code of
#'     that program is returned as the student's FYE proxy.}
#' \item{The student does not enroll post-FYE in an engineering major. Such
#'     students have no further records in the database or switched from
#'     Engineering to another program. For this outcome, the data provide no
#'     information regarding what engineering program the student would have
#'     declared originally had the institution not required them to enroll in
#'     FYE. For these students a proxy value of NA is returned. These are the
#'     data treated as missing values to be imputed by `mice()`.}
#' }
#'
#' In cases where students enter FYE, change programs, and re-enter FYE, only
#' the first group of FYE terms is considered. Any programs before FYE are
#' ignored.
#'
#' The resulting data frame is ready for use as input for the mice package,
#' with all variables except `mcid` returned as factors.
#'
#' @example man/examples/prep_fye_mice_exa.R
#' @export
#'
prep_fye_mice <- function(midfield_student = student,
                          midfield_term = term,
                          ...,
                          fye_codes = NULL) {
  # remove all keys
  on.exit(setkey(midfield_student, NULL))
  on.exit(setkey(midfield_term, NULL), add = TRUE)

  # assert arguments after dots used by name
  wrapr::stop_if_dot_args(
    substitute(list(...)),
    paste(
      "Arguments after ... must be named.\n",
      "* Did you forget to write `fye_codes = `?\n *"
    )
  )

  # optional arguments: fye_codes default value(s)
  fye_codes <- fye_codes %?% c("140102")

  # required arguments
  qassert(midfield_student, "d+")
  qassert(midfield_term, "d+")
  qassert(unique(nchar(fye_codes)), "I1[6,6]")

  # fye_codes: number strings only, no regular expressions
  assert_subset(
    unlist(strsplit(fye_codes, split = character(0))),
    choices = c("0", "1", "2", "3", "4", "5", "6", "7", "8", "9")
  )

  # fye_codes: string, length >= 1
  qassert(fye_codes, "s+")

  # fye_codes: must be engineering (start with "14")
  assert_subset(
    substr(fye_codes, 1, 2),
    choices = c("14")
  )

  # inputs modified (or not) by reference
  setDT(midfield_student) # immediately subset, so side-effect OK
  setDT(midfield_term) # immediately subset, so side-effect OK

  # required columns
  assert_names(colnames(midfield_student),
    must.include = c("mcid", "race", "sex")
  )
  assert_names(colnames(midfield_term),
    must.include = c("mcid", "institution", "term", "cip6")
  )

  # class of required columns
  qassert(midfield_student[, mcid], "s+")
  qassert(midfield_student[, race], "s+")
  qassert(midfield_student[, sex], "s+")
  qassert(midfield_term[, mcid], "s+")
  qassert(midfield_term[, institution], "s+")
  qassert(midfield_term[, term], "s+")
  qassert(midfield_term[, cip6], "s+")

  # bind names due to NSE notes in R CMD check
  proxy <- NULL

  # Do the work

  # All FYE students, all terms
  fye <- midfield_term[cip6 %chin% fye_codes, .(mcid, institution)]
  on.exit(setkey(fye, NULL), add = TRUE)

  # Limit to degree-seeking IDs (inner join)
  # The fye data frame is the function output after we add
  # the FYE proxy CIP variable, institution, race, and sex
  fye <- midfield_student[fye, .(mcid, institution), on = c("mcid"), nomatch = NULL]
  fye <- unique(fye)

  # Working data frame to gather proxies (left-outer join)
  DT <- midfield_term[fye, .(mcid, institution, term, cip6), on = c("mcid")]
  on.exit(setkey(DT, NULL), add = TRUE)
  DT <- unique(DT)

  # Order rows and create proxy variable as CIP in following term
  setkeyv(DT, c("mcid", "term"))
  DT[, proxy := shift(.SD, n = 1, fill = NA, type = "lead"),
    by = "mcid",
    .SDcols = "cip6"
  ]

  # Omit rows for which the proxy is FYE, retaining
  # rows with student's last FYE term
  DT <- DT[!proxy %chin% fye_codes]

  # Omit rows in which consecutive CIPs are identical
  DT <- DT[cip6 != proxy]

  # Keep rows in which the term cip6 is FYE
  DT <- DT[cip6 %chin% fye_codes]

  # Ensure row order, keep the first instance by ID, thereby
  # omitting rows for students entering FYE twice
  setkeyv(DT, c("mcid", "term"))
  DT <- DT[, .SD[1], by = c("mcid")]

  # subset to retain those who transition to engr major after FYE
  DT <- DT[proxy %like% "^14"]

  # Drop unnecessary columns. This DT contains the known CIPs of
  # FYE students who transition to an ENGR major post-FYE
  DT <- DT[, .(mcid, proxy)]

  # Merge known transition CIPs to ever FYE (left-outer join)
  fye <- DT[fye, on = c("mcid")]

  # Add race and sex (left-outer join). Ensure uniqueness.
  fye <- midfield_student[fye, .(mcid, race, sex, institution, proxy), on = c("mcid")]
  fye <- unique(fye)

  # Convert to factors to prepare for mice()
  fye[, `:=`(
    race = as.factor(race),
    sex = as.factor(sex),
    institution = as.factor(institution),
    proxy = as.factor(proxy)
  )]

  # reorder rows
  setkeyv(fye, c("institution", "proxy", "sex", "race"))

  # enable printing (see data.table FAQ 2.23)
  fye[]
}

test_prep_fye_mice <- function() {

    # usage
    # prep_fye_mice(midfield_student,   # mcid, race, sex
    #               midfield_term,      # term, institution
    #               ...,
    #               fye_codes = NULL)   # default 140102

    # Needed for tinytest::build_install_test()
    data("toy_student")
    data("toy_term")
    library("data.table")
    options(
        datatable.print.nrows = 8,
        datatable.print.topn = 8,
        datatable.print.class = TRUE
    )

    # create an answer
    # DT <- prep_fye_mice(toy_student, toy_term)
    # cat(wrapr::draw_frame(DT))

    # test case
    DT <- wrapr::build_frame(
        "mcid"         , "race"  , "sex"   , "institution"  , "proxy"       |
            "MID26060301", "Asian" , "Female", "Institution C", NA_character_ |
            "MID25995980", "Hispanic", "Female", "Institution C", NA_character_ |
            "MID25997636", "Hispanic", "Female", "Institution C", NA_character_ |
            "MID26086310", "Hispanic", "Female", "Institution C", NA_character_ |
            "MID26000057", "White" , "Female", "Institution C", NA_character_ |
            "MID26614720", "Asian" , "Male"  , "Institution J", NA_character_ |
            "MID26593796", "White" , "Male"  , "Institution J", NA_character_ |
            "MID25848589", "White" , "Male"  , "Institution M", "143501"      |
            "MID25846316", "White" , "Male"  , "Institution M", "143501"      |
            "MID25847220", "White" , "Male"  , "Institution M", "143501"      |
            "MID25828870", "White" , "Male"  , "Institution M", "149999"      )
    setDT(DT)
    DT[, c("race", "sex", "institution", "proxy") :=
           list(as.factor(race),
                as.factor(sex),
                as.factor(institution),
                as.factor(proxy)
           )]
    # DT[]

    # Correct answer
    expect_equal(
        DT,
        prep_fye_mice(toy_student, toy_term)
    )

    # Results are factors except for ID
    DT <- prep_fye_mice(toy_student, toy_term)
    expect_equal(class(DT[, mcid]), "character")
    expect_equal(class(DT[, institution]), "factor")
    expect_equal(class(DT[, race]), "factor")
    expect_equal(class(DT[, sex]), "factor")
    expect_equal(class(DT[, proxy]), "factor")

    # specific names of columns
     expect_equivalent(
        names(DT),
        c("mcid", "race", "sex", "institution", "proxy")
    )

     # extra columns are dropped, add column for cip6
     DT <- prep_fye_mice(toy_student, toy_term)
     expect_equal(
         names(DT),
         c("mcid", "race", "sex", "institution", "proxy")
     )

    # CIPs must be 6-digit, number characters only, start with 14
    x <- toy_student[, .(mcid, race, sex)]
    expect_error(
        prep_fye_mice(x, toy_term, fye_codes = c("14", "1410", "143501"))
    )
    expect_error(
        prep_fye_mice(x, toy_term, fye_codes = c("^14350", "143501"))
    )
    expect_error(
        prep_fye_mice(x, toy_term, fye_codes = c("543501", "143501"))
    )

    # multiple CIP codes starting with 14 can be assigned to FYE
    # create an answer
    # DT <- prep_fye_mice(toy_student, toy_term,
    #                     fye_codes = c("140101", "140102", "149999"))
    # cat(wrapr::draw_frame(DT))
    DT <- wrapr::build_frame(
        "mcid"         , "race"  , "sex"   , "institution"  , "proxy"       |
            "MID25977316", "White" , "Male"  , "Institution B", NA_character_ |
            "MID26060301", "Asian" , "Female", "Institution C", NA_character_ |
            "MID25995980", "Hispanic", "Female", "Institution C", NA_character_ |
            "MID25997636", "Hispanic", "Female", "Institution C", NA_character_ |
            "MID26086310", "Hispanic", "Female", "Institution C", NA_character_ |
            "MID26000057", "White" , "Female", "Institution C", NA_character_ |
            "MID26231601", "White" , "Female", "Institution E", NA_character_ |
            "MID26171165", "Hispanic", "Male"  , "Institution E", NA_character_ |
            "MID26614720", "Asian" , "Male"  , "Institution J", NA_character_ |
            "MID26593796", "White" , "Male"  , "Institution J", NA_character_ |
            "MID25828870", "White" , "Male"  , "Institution M", "140801"      |
            "MID25848589", "White" , "Male"  , "Institution M", "143501"      |
            "MID25846316", "White" , "Male"  , "Institution M", "143501"      |
            "MID25847220", "White" , "Male"  , "Institution M", "143501"      )
    setDT(DT)
    DT[, c("race", "sex", "institution", "proxy") :=
           list(as.factor(race),
                as.factor(sex),
                as.factor(institution),
                as.factor(proxy)
           )]

    # Correct answer
    expect_equal(
        DT,
        prep_fye_mice(toy_student, toy_term, fye_codes = c("140101", "140102", "149999"))
    )
    
    # CIP must start with 14
    expect_error(
        prep_fye_mice(toy_student, toy_term, fye_codes = c("140101", "140102", "999999"))
    )

    invisible(NULL)
}

test_prep_fye_mice()

# Start with an excerpt from the student data set 
dframe <- toy_student[1:10, .(mcid)]

# Timely term column is required to add data sufficiency column
dframe <- add_timely_term(dframe, midfield_term = toy_term)

# Add data sufficiency column
add_data_sufficiency(dframe, midfield_term = toy_term)

# Existing data_sufficiency column, if any, is overwritten
dframe[, data_sufficiency := NA_character_]
add_data_sufficiency(dframe, midfield_term = toy_term)

# Start with an excerpt from the student data set 
dframe <- toy_student[1:10, .(mcid)]

# Timely term column is required to add completion status column
dframe <- add_timely_term(dframe, toy_term)

# Add completion status column
add_completion_status(dframe, toy_degree)

# Existing completion_status column, if any, is overwritten
dframe[, completion_status := NA_character_]
add_completion_status(dframe, toy_degree)

# R code used for documentation

dframe_add_completion_status <- "Working data frame of student-level records
        to which completion-status columns are to be added. Required variables
        are `mcid` and `timely_term.` See also `add_timely_term()`."

dframe_add_data_sufficiency <- "Working data frame of student-level records
        to which data-sufficiency columns are to be added. Required variables
        are `mcid` and `timely_term.` See also `add_timely_term()`."

dframe_add_timely_term <- "Working data frame of student-level records
        to which timely-term columns are to be added. Required variable
        is `mcid`."

dframe_order_multiway <- 'Data frame containing a single quantitative value
        (or response) for every combination of levels of two categorical
        variables. Categories may be class character or factor. Two additional
        numeric columns are required when using the "percent" ordering method.'


midfield_student_prep_fye_mice <- "MIDFIELD `student` data table or
        equivalent with required variables `mcid`, `race`, and `sex.`"

midfield_term_add_data_sufficiency <- "MIDFIELD `term` data table or equivalent
        with required variables `mcid`, `institution`, and `term`."

midfield_term_add_timely_term <- "MIDFIELD `term` data table or equivalent
        with required variables `mcid`, `term`, and `level.`"

midfield_term_prep_fye_mice <- "MIDFIELD `term` data table or equivalent
        with required variables `mcid`, `institution`, `term`, and `cip6.`"

midfield_degree_add_completion_status <- "MIDFIELD `degree` data table or
        equivalent with required variables `mcid` and `term_degree.`"


param_dots <- "Not used for passing values; forces subsequent arguments to be
        referable only by name."


return_data_frame <- "A data frame in `data.table` format with the following
        properties: rows are preserved; columns are preserved with the exception
        that columns added by the function overwrite existing columns of the
        same name (if any); grouping structures are not preserved. The added
        columns are:"

return_order_multiway <- "A data frame in `data.table` format with
        the following properties: rows are preserved; columns specified by
        `categories` are converted to factors and ordered; the column specified
        by `quantity` is converted to type double; other columns are preserved
        with the exception that columns added by the function overwrite existing
        columns of the same name (if any); grouping structures are not preserved.
        The added columns are:"

return_filter_cip <- "A data frame in `data.table` format, a subset of `cip`,
        with the following properties: exclude rows that match
        elements of `drop_text`; of the remaining rows, include those that
        match elements of `keep_text`; if `select` is empty, all columns are
        preserved, otherwise only columns included in `select` are retained;
        grouping structures are not preserved."

return_select_required <- "A data frame in `data.table` format with the
        following properties: rows are preserved; columns with names that
        match or partially match values in the default set plus any in
        `select_add` are retained; grouping structures are not preserved."

return_prep_fye_mice <- "A data frame in `data.table` format conditioned for
        later use as an input to the mice R package for multiple imputation. The
        data frame comprises one row for every FYE student, first-term and
        migrator. Grouping structures are not preserved. The columns returned
        are:"


# Variable names in data sets

var_abbrev <- "\\item{`abbrev`}{Character, course alphabetical identifier,
                  e.g. ENGR, MATH, ENGL. Key column.}"
var_bloc <- "\\item{`bloc`}{Character, indicating the grouping
                  (`ever_enrolled` or `graduates`) to which an observation
                  belongs.}"
var_cip6 <- "\\item{`cip6`}{Character, 6-digit CIP code of program in which
                  a student is enrolled in a term.}"
var_ever_enrolled <- "\\item{`ever_enrolled`}{Numerical, number of students
                  ever enrolled in a program.}"
var_grade <- "\\item{`grade`}{Character, course grade, e.g., A+, A, A-, B+,
                  I, NG, etc.}"
var_graduates <- "\\item{`graduates`}{Numerical, number of students completing
                  a program.}"
var_institution <- "\\item{`institution`}{Character, de-identified institution
                  name, e.g., Institution A, Institution B, etc.}"
var_level <- "\\item{`level`}{Character, 01 Freshman, 02 Sophomore, etc.
                  The equivalent values in the current practice data are 01
                  First-Year, 02-Second Year, etc.}"
var_mcid <- "\\item{`mcid`}{Character, de-identified student ID. Key column.}"
var_number <- "\\item{`number`}{Character, course numeric identifier, e.g.
                  101, 3429. Key column.}"
var_program <- "\\item{`program`}{Character, academic program label.}"
var_proxy <- "\\item{`proxy`}{Character, 6-digit CIP code of the estimated
                  proxy program.}"
var_race <- "\\item{`race`}{Character, race/ethnicity as self-reported
                  by the student, e.g., Asian, Black, Hispanic, etc.}"
var_sex <- "\\item{`sex`}{Character, sex as self-reported by the student,
                  possible values are Female, Male, and Unknown.}"
var_term <- "\\item{`term`}{Character, academic year and term, format YYYYT.
                  Key column.}"
var_term_degree <- "\\item{`term_degree`}{Character, academic year and term
                  in which a student completes their program, format YYYYT.}"
var_stickiness <- "\\item{`stickiness`}{Numerical, program stickiness, the
                  ratio of `graduates` to `ever_enrolled`, in percent.}"

test_check_equiv_frames <- function() {

    # usage
    # check_equiv_frames(x, y)

    # Needed for tinytest::build_install_test()
    library("data.table")

    # test cases
    alpha <- letters[1:5]
    beta <- 5:1
    dt1 <- data.table(alpha, beta)
    
    gamma <- LETTERS[1:10]
    delta <- 10:1
    epsilon <- runif(10, 1, 100)
    dt2 <- data.table(gamma, delta, epsilon)
    
    # Same information but different row order, column order, and key
    x <- dt1[, .(alpha, beta)]
    setkey(x, alpha)
    y <- dt1[, .(beta, alpha)]
    setkey(y, beta)
    expect_true(check_equiv_frames(x, y))
    
    # When not same content, output is FALSE
    expect_true(!check_equiv_frames(dt1, dt2))
    
    # Different number of rows only
    expect_true(!check_equiv_frames(dt1, dt1[1:4]))
    
    # Different column names only
    expect_true(!check_equiv_frames(dt2[, .(gamma, delta)], dt2[, .(gamma, epsilon)]))
    
    # Different number of columns
    expect_true(!check_equiv_frames(dt2, dt2[, 1:2]))

    # Same number of rows, different row content
    expect_true(!check_equiv_frames(dt1[1:4], dt1[2:5]))

    # data frame arguments must be data frames
    expect_true(!check_equiv_frames(dt1$alpha, dt1))
    
    # function output not printed
    invisible(NULL)
}

test_check_equiv_frames()

undecided/unspecified

: The MIDFIELD taxonomy includes the non-IPEDS code (CIP 999999) for Undecided or Unspecified indicating instances in which a student has not declared a major or an institution had not recorded a program. 

multiway chart

: A multi-panel dot plot: horizontal, quantitative scales; rows that encode one category; and panels that encode the second category. All panels have identical axes. The ordering of the rows and panels is crucial to the perception of effects.

---
title: "midfieldr"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{midfieldr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Please see the midfieldr website for full documentation:

  * <https://midfieldr.github.io/midfieldr/>
  
The website also includes articles describing typical applications in detail: 

  * [Index of articles](https://midfieldr.github.io/midfieldr/articles/index.html)
  

CIP

:  *Classification of Instructional Programs*, a taxonomy of academic programs curated by the US Department of Education [@NCES:2010]. The 2010 codes are included with midfieldr in the data set `cip`. 

timely completion term

:   The last term in which a student's degree completion would be considered timely. In many cases the timely completion (TC) term is 6 years after admission. The TC term can be adjusted to account for transfer credits. (Currently, there is no mechanism for extending the TC term for co-ops or migrators.) 

inner join

: A merge operation between two data frames `X` and `Y` that returns observations (rows) that match specified conditions in both. The data.table syntax is `Y[X, j, on]` where `j` can be used to select columns. 

```{r child-preparing-data-setup}
#| include: true

# Packages
library(midfieldr)    # check_equiv_frames()
library(midfielddata) # datasets
library(data.table)   # data manipulation system
library(dplyr)        # data manipulation system
library(tidyr)        # drop_na()

# Initialize data frames 
data(student, term)
student_df  <- data.frame(student)
student_dt  <- copy(student)
student_tbl <- tibble(student)
term_df  <- data.frame(term)
term_dt  <- copy(term)
term_tbl <- tibble(term)
```

[MIDFIELD](https://midfield.online/). &nbsp; A database of anonymized student-level records for approximately 2.4M undergraduates at 21 US institutions from 1987-2022. In 2023, control and management of the database was transferred to the American Society for Engineering Education (ASEE). For further information, contact ASEE.  

[midfielddata](https://midfieldr.github.io/midfielddata/). &nbsp; An R data package that supplies anonymized student-level records for 98,000 undergraduates at three US institutions from 1988-2018. A sample of the MIDFIELD database, midfielddata provides practice data for the tools and methods in the midfieldr package.

```{r}
#| echo: false

# Restore the user options (saved in common-setup.Rmd)
options(backup_options)

# Restore user random number seed if any
if (!is.null(oldseed)) {.Random.seed <- oldseed}

# to change the CSS file
# per https://github.com/rstudio/rmarkdown/issues/732
knitr::opts_chunk$set(echo = FALSE)
```


<!-- blockquote { -->
<!--     padding:     10px 20px; -->
<!--     margin:      0 0 20px; -->
<!--     border-left: 0px -->
<!-- } -->
<!-- caption { -->
<!--     color:       #525252; -->
<!--     text-align:  left; -->
<!--     font-weight: normal; -->
<!--     font-size:   medium; -->
<!--     line-height: 1.5; -->
<!-- } -->

As of the transfer of MIDFIELD to the American Society for Engineering 
Education (ASEE) in 2023, the development, expansion, and study of 
MIDFIELD has been supported by the National Science Foundation grants 
0337629, 0646441, 0729596, 0734062, 0835914, 0935157, 0935058, 
0969474, 1025171, 1129383, 1232740, 1329283, 1361058, 1545667, 
2142087, 2141903, and 2152441.

FYE

: First-Year Engineering program, a common-first-year curriculum that is a prerequisite for declaring an engineering major at some US institutions. Denoted by its own CIP code, FYE is not a degree-granting program. 

FYE proxy

: Our estimate of the degree-granting engineering program in which an FYE student would have enrolled had they not been required to enroll in FYE. The proxy, a 6-digit CIP code, denotes the program of which the FYE student can be considered a starter.

student-level data

: Data at the "student-level" refers to information about individual students including, for example, demographics, programs, academic standing, courses, grades, and degrees. Also called Student Unit Records (SURs). 

: MIDFIELD student-level data are provided in four data tables (`student`, `course`, `term`, and `degree`) that were compiled by institutions and anonymized and curated by the MIDFIELD data steward.   

stickiness

: Program "stickiness" $\small\left(S\right)$ is the ratio of the number of graduates of a program $\small\left(N_g\right)$ to the number ever enrolled in the program $\small\left(N_e\right)$.

grouping variables

: Detailed information in the student-level data that further characterize a bloc of records, typically used to create bloc subsets for comparison, for example, program, race/ethnicity, sex, age, grade level, grades, etc. 

*Note.* &nbsp; MIDFIELD research findings are regularly grouped by program, race/ethnicity, and sex. However, applied to the practice data these groupings produce several groups with totals below the threshold we impose to preserve anonymity, introducing a number of NA values in the resulting charts and tables. These NAs are largely an artifact of applying these groupings to practice data.

*Reminder.* &nbsp; midfielddata is suitable for learning to work with student-level data but not for drawing inferences about program attributes or student experiences. midfielddata supplies practice data, not research data. 

multiway data

: A data set of three variables: a category with _m_ levels; a second independent category with _n_ levels; and a quantitative variable (the response) of length _mn_ such that there is a value of the response for each combination of levels of the two categorical variables.

*Initialize.* &nbsp; Use the `term` and `student` data tables to obtain a data frame of student IDs meeting the data sufficiency and degree-seeking criteria. Appled to the practice data, this procedure yields the `baseline_mcid` data frame derived in [Blocs](art-050-blocs.html#initial-processing) and included with midfieldr. 

timely completion criterion

: Completing a program in no more than a specified span of years, in many cases, within 6 years after admission (150% of the "normal" 4-year span), or possibly less for some transfer students.

metric

: A quantitative measure derived from student-level data. Includes statistical measures such as counts of program starters or graduates as well as comparative ratios such as graduation rate or stickiness. Typically involves comparisons of specific  blocs of students and programs. 

left join

: A merge operation between two data frames `X` and `Y` which returns all observations (rows) of `X` and all matching rows in `Y`. The data.table syntax is `Y[X, j, on]` where `j` can be used to select columns.

graduation rate

: Graduation rate $(G)$ is the ratio of the number of program "starter-graduates" $(N_{sg})$ (i.e., graduates from the program in which they started) to the number of program starters $(N_s)$.

```{r child-common-setup}
#| include: false
# code chunks
knitr::opts_chunk$set(fig.width = 8,
                      out.width = "100%",
                      collapse  = TRUE, 
                      comment   = "#>",
                      message   = FALSE, 
                      cache     = FALSE, 
                      error     = FALSE,
                      tidy      = FALSE, 
                      echo      = TRUE)

# inline numbers
knitr::knit_hooks$set(inline = function(x) {
    if (!is.numeric(x)) {
        x
    } else if (x >= 10000) {
        prettyNum(round(x, 2), big.mark = ",")
    } else {
        prettyNum(round(x, 2))
    }
})

# Backup user options (load packages to capture default options)
suppressPackageStartupMessages(library(data.table))
backup_options <- options()

# Backup user random number seed
oldseed <- NULL
if (exists(".Random.seed")) oldseed <- .Random.seed

# data.table printout
options(datatable.print.nrows = 10,
        datatable.print.topn = 3,
        datatable.print.class = TRUE)
```

program

: US academic field of study. Can be used to indicate a specialty within a field or a collection of fields within a Department, College, or University. Programs are denoted by the *Classification of Instructional Programs* (CIP), a taxonomy of academic programs curated by the US Department of Education [@NCES:2010].

Provides tools and demonstrates methods for working with individual undergraduate student-level records (registrar's data) in R. Tools include filters for program codes, data sufficiency, and timely completion. Methods include gathering blocs of records, computing quantitative metrics such as graduation rate, and creating charts to visualize comparisons. midfieldr is designed to work with data from the MIDFIELD research database, a sample of which is available in the midfielddata data package.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{study_results}
\alias{study_results}
\title{Case-study results}
\format{
\code{data.table} with 50 rows and 6 columns:
\describe{
\item{\code{program}}{Character, academic program label.}
\item{\code{sex}}{Character, sex as self-reported by the student,
possible values are Female, Male, and Unknown.}
\item{\code{race}}{Character, race/ethnicity as self-reported
by the student, e.g., Asian, Black, Hispanic, etc.}
\item{\code{ever_enrolled}}{Numerical, number of students
ever enrolled in a program.}
\item{\code{graduates}}{Numerical, number of students completing
a program.}
\item{\code{stickiness}}{Numerical, program stickiness, the
ratio of \code{graduates} to \code{ever_enrolled}, in percent.}
}
}
\usage{
data(study_results)
}
\description{
Data table of longitudinal stickiness for the four programs of the case study
(Civil, Electrical, Industrial/Systems, and Mechanical Engineering) grouped
by program, race/ethnicity, and sex. Provided for the convenience of vignette
users.
}
\details{
Longitudinal stickiness is the ratio of the number of students graduating
from a program to the number of students ever enrolled in the program over
the time span of available data. Results are based on data that have been
filtered for data sufficiency, degree seeking, and timely completion.
}
\seealso{
Other case-study-data:
\code{\link{baseline_mcid}},
\code{\link{study_observations}},
\code{\link{study_programs}}
}
\concept{case-study-data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_cip.R
\name{filter_cip}
\alias{filter_cip}
\title{Subset rows that include matches to search strings}
\usage{
filter_cip(keep_text = NULL, ..., drop_text = NULL, cip = NULL, select = NULL)
}
\arguments{
\item{keep_text}{Character vector of search text for retaining rows,
not case-sensitive. Can be empty if \code{drop_text} is used.}

\item{...}{Not used for passing values; forces subsequent arguments to be
referable only by name.}

\item{drop_text}{Optional character vector of search text for dropping
rows, default NULL.}

\item{cip}{Data frame to be searched. Default \code{cip}.}

\item{select}{Optional character vector of column names to return,
default all columns.}
}
\value{
A data frame in \code{data.table} format, a subset of \code{cip},
with the following properties: exclude rows that match
elements of \code{drop_text}; of the remaining rows, include those that
match elements of \code{keep_text}; if \code{select} is empty, all columns are
preserved, otherwise only columns included in \code{select} are retained;
grouping structures are not preserved.
}
\description{
Subset a CIP data frame, retaining rows that match or partially match a
vector of character strings. Columns are not subset unless selected in an
optional argument.
}
\details{
Search terms can include regular expressions. Uses \code{grepl()}, therefore
non-character columns (if any) that can be coerced to character are also
searched for matches. Columns are subset by the values in \code{select} after the
search concludes.

If none of the optional arguments are specified, the function returns the
original data frame.
}
\examples{
# Subset using keywords
filter_cip(keep_text = "engineering")

\donttest{
    # Multiple passes to narrow the results
    first_pass <- filter_cip("civil")
    second_pass <- filter_cip("engineering", cip = first_pass)
    filter_cip(drop_text = "technology", cip = second_pass)
    
    # drop_text argument, when used, must be named
    filter_cip("civil engineering", drop_text = "technology")
    
    # Subset using numerical codes
    filter_cip(keep_text = c("050125", "160501"))
    
    # Subset using regular expressions
    filter_cip(keep_text = "^54")
    filter_cip(keep_text = c("^1407", "^1408"))
    
    # Select columns
    filter_cip(keep_text = "^54", select = c("cip6", "cip4name"))
}
}
\concept{filter_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{toy_student}
\alias{toy_student}
\title{Student data for examples}
\format{
\code{data.table} with 99 rows and 4 columns keyed by student ID:
\describe{
\item{\code{mcid}}{Character, de-identified student ID. Key column.}
\item{\code{institution}}{Character, de-identified institution
name, e.g., Institution A, Institution B, etc.}
\item{\code{race}}{Character, race/ethnicity as self-reported
by the student, e.g., Asian, Black, Hispanic, etc.}
\item{\code{sex}}{Character, sex as self-reported by the student,
possible values are Female, Male, and Unknown.}
}
}
\usage{
data(toy_student)
}
\description{
Selected variables modeled on those in the \code{student} practice data for use in
package examples and articles. Sampled from an early version of the practice
data, the toy data are not a current practice data sample.
}
\seealso{
Other toy-data:
\code{\link{toy_course}},
\code{\link{toy_degree}},
\code{\link{toy_term}}
}
\concept{toy-data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{toy_term}
\alias{toy_term}
\title{Term data for examples}
\format{
\code{data.table} with 150 rows and 5 columns keyed by student ID.
The variables are:
\describe{
\item{\code{mcid}}{Character, de-identified student ID. Key column.}
\item{\code{institution}}{Character, de-identified institution
name, e.g., Institution A, Institution B, etc.}
\item{\code{term}}{Character, academic year and term, format YYYYT.
Key column.}
\item{\code{cip6}}{Character, 6-digit CIP code of program in which
a student is enrolled in a term.}
\item{\code{level}}{Character, 01 Freshman, 02 Sophomore, etc.
The equivalent values in the current practice data are 01
First-Year, 02-Second Year, etc.}
}
}
\usage{
data(toy_term)
}
\description{
Selected variables modeled on those in the \code{term} practice data for use in
package examples and articles. Sampled from an early version of the practice
data, the toy data are not a current practice data sample.
}
\seealso{
Other toy-data:
\code{\link{toy_course}},
\code{\link{toy_degree}},
\code{\link{toy_student}}
}
\concept{toy-data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_data_sufficiency.R
\name{add_data_sufficiency}
\alias{add_data_sufficiency}
\title{Determine data sufficiency for every student}
\usage{
add_data_sufficiency(dframe, midfield_term = term)
}
\arguments{
\item{dframe}{Working data frame of student-level records
to which data-sufficiency columns are to be added. Required variables
are \code{mcid} and \code{timely_term.} See also \code{add_timely_term()}.}

\item{midfield_term}{MIDFIELD \code{term} data table or equivalent
with required variables \code{mcid}, \code{institution}, and \code{term}.}
}
\value{
A data frame in \code{data.table} format with the following
properties: rows are preserved; columns are preserved with the exception
that columns added by the function overwrite existing columns of the
same name (if any); grouping structures are not preserved. The added
columns are:
\describe{
\item{\code{term_i}}{Character. Initial term of a student's longitudinal
record, encoded YYYYT. Not overwritten if present in \code{dframe.}}
\item{\code{lower_limit}}{Character. Initial term of an institution's data
range, encoded YYYYT}
\item{\code{upper_limit}}{Character. Final term of an institution's data
range, encoded YYYYT}
\item{\code{data_sufficiency}}{Character. Label each observation for
inclusion or exclusion based on data sufficiency. Possible values are:
\code{include}, indicating that available data are sufficient for
estimating timely completion; \code{exclude-upper}, indicating
that data are insufficient at the upper limit of a data range; and
\code{exclude}-lower, indicating that data are insufficient at the
lower limit.}
}
}
\description{
Add a column to a data frame of student-level records that labels each row
for inclusion or exclusion based on data sufficiency near the upper and lower
bounds of an institution's data range.
}
\details{
The time span of MIDFIELD term data varies by institution, each having their
own lower and upper bounds. For some student records, being at or near these
bounds creates unavoidable ambiguity when trying to assess degree completion.
Such records must be identified and in most cases excluded to prevent false
summary counts.
}
\examples{
# Start with an excerpt from the student data set 
dframe <- toy_student[1:10, .(mcid)]

# Timely term column is required to add data sufficiency column
dframe <- add_timely_term(dframe, midfield_term = toy_term)

# Add data sufficiency column
add_data_sufficiency(dframe, midfield_term = toy_term)

# Existing data_sufficiency column, if any, is overwritten
dframe[, data_sufficiency := NA_character_]
add_data_sufficiency(dframe, midfield_term = toy_term)
}
\seealso{
Other add_*:
\code{\link[=add_completion_status]{add_completion_status()}},
\code{\link[=add_timely_term]{add_timely_term()}}
}
\concept{add_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cip}
\alias{cip}
\title{Table of academic programs}
\format{
\code{data.table} with 1582 rows and 6 columns keyed by the 6-digit CIP
code:

\describe{

\item{\code{cip6}}{Character 6-digit code representing "specific
instructional programs" (US National Center for Education Statistics).}

\item{\code{cip6name}}{Character program name at the 6-digit level}

\item{\code{cip4}}{Character 4-digit code (the first 4 digits of \code{cip6})
representing "intermediate groupings of programs that have
comparable content and objectives."}

\item{\code{cip4name}}{Character program name at the 4-digit level.}

\item{\code{cip2}}{Character 2-digit code (the first 2 digits of \code{cip6})
representing "the most general groupings of related programs."}

\item{\code{cip2name}}{Character program name at the 2-digit level.}

}
}
\usage{
data(cip)
}
\description{
A data table based on the US National Center for Education Statistics (NCES),
Integrated Postsecondary Education Data System (IPEDS), 2010 CIP,
\url{http://nces.ed.gov/ipeds/cipcode/}. The data are codes and names for
1582 instructional programs organized on three levels: a 2-digit series, a
4-digit series, and a 6-digit series.
}
\details{
The midfielddata taxonomy includes one non-IPEDS code (999999) for Undecided
or Unspecified, instances in which institutions reported no program
information or that students were not enrolled in a program.

The MIDFIELD research database include CIPs for undergraduate pre-majors
such as pre-med (511102), pre-law (220001), and pre-vet (511104).
}
\seealso{
Other cip-data:
\code{\link{fye_proxy}}
}
\concept{cip-data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_timely_term.R
\name{add_timely_term}
\alias{add_timely_term}
\title{Calculate a timely completion term for every student}
\usage{
add_timely_term(
  dframe,
  midfield_term = term,
  ...,
  span = NULL,
  sched_span = NULL
)
}
\arguments{
\item{dframe}{Working data frame of student-level records
to which timely-term columns are to be added. Required variable
is \code{mcid}.}

\item{midfield_term}{MIDFIELD \code{term} data table or equivalent
with required variables \code{mcid}, \code{term}, and \code{level.}}

\item{...}{Not used for passing values; forces subsequent arguments to be
referable only by name.}

\item{span}{Optional integer scalar, number of years to define timely
completion. Commonly used values are are 100, 150, and 200 percent of
\code{sched_span.} Default 6 years.}

\item{sched_span}{Optional integer scalar, the number of years an institution
officially schedules for completing a program. Default 4 years.}
}
\value{
A data frame in \code{data.table} format with the following
properties: rows are preserved; columns are preserved with the exception
that columns added by the function overwrite existing columns of the
same name (if any); grouping structures are not preserved. The added
columns are:
\describe{
\item{\code{term_i}}{Character. Initial term of a student's longitudinal
record, encoded YYYYT}
\item{\code{level_i}}{Character. Student level (01 Freshman, 02 Sophomore,
etc.) in their initial term}
\item{\code{adj_span}}{Numeric. Integer span of years for timely completion
adjusted for a student's initial level.}
\item{\code{timely_term}}{Character. Latest term by which program
completion would be considered timely for every student. Encoded YYYYT.}
}
}
\description{
Add a column to a data frame of student-level records that indicates the
latest term by which degree completion  would be considered timely for every
student.
}
\details{
By "completion" we mean an undergraduate earning their first baccalaureate
degree (or degrees, for students earning more than one degree in the same
term).

In many studies, students must complete their programs in a specified time
span, for example 4-, 6-, or 8-years after admission. If they do, their
completion is timely; if not, their completion is late and they are grouped
with the non-completers when computing a metric such as graduation rate.

Our heuristic assigns \code{span} number of years (default is 6 years) to every
student. For students admitted at second-year level or higher, the span is
reduced by one year for each full year the student is assumed to have
completed. For example, a student admitted at the second-year level is
assumed to have completed one year of a program, so their span is reduced by
one year.

The adjusted span is added to the initial term to create the timely
completion term in the \code{timely_term} column.
}
\examples{
# Start with an excerpt from the student data set 
dframe <- toy_student[1:10, .(mcid)]

# Add timely completion term column
add_timely_term(dframe, midfield_term = toy_term)

# Define timely completion as 200\% of scheduled span (8 years)
add_timely_term(dframe, midfield_term = toy_term, span = 8)

# Existing timely_term column, if any, is overwritten
dframe[, timely_term := NA_character_]
add_timely_term(dframe, midfield_term = toy_term)


}
\seealso{
Other add_*:
\code{\link[=add_completion_status]{add_completion_status()}},
\code{\link[=add_data_sufficiency]{add_data_sufficiency()}}
}
\concept{add_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{grade_scale}
\alias{grade_scale}
\title{Grade scale}
\format{
\code{data.table} with 12 rows and 2 columns:
\describe{
\item{\code{letter_grade}}{Character, letter grades using the conventional US
scale from A to F.}
\item{\code{points}}{Numerical, 4.0 scale of points assigned to letter grades.}
}
}
\usage{
data(grade_scale)
}
\description{
Data frame of letter grades and conventional point assignments used for
computing grade point averages.
}
\seealso{
Other scales:
\code{\link{act_sat_scale}}
}
\concept{scales}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{baseline_mcid}
\alias{baseline_mcid}
\title{Baseline ID bloc to start a typical analysis}
\format{
\code{data.table} with 76875 rows and 1 column:
\describe{
\item{\code{mcid}}{Character, de-identified student ID. Key column.}
}
}
\usage{
data(baseline_mcid)
}
\description{
Data frame of IDs after processing the practice data for data sufficiency and
degree seeking. Provides a convenient bloc to start many of the analysis
illustrated in the package articles.
}
\seealso{
Other case-study-data:
\code{\link{study_observations}},
\code{\link{study_programs}},
\code{\link{study_results}}
}
\concept{case-study-data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{act_sat_scale}
\alias{act_sat_scale}
\title{ACT-SAT conversion scale}
\format{
\code{data.table} with 28 rows and 4 columns:
\describe{
\item{\code{act_comp}}{Numerical, ACT composite score.}
\item{\code{sat_lower}}{Numerical, total SAT, lower limit of range
corresponding to the ACT composite score.}
\item{\code{sat_equiv}}{Numerical, total SAT, value to use when
converting ACT score to a single SAT score.}
\item{\code{sat_upper}}{Numerical, total SAT, upper limit of range
corresponding to the ACT composite score.}
}
}
\source{
ACT/SAT Concordance (2018) ACT Education Corp. \url{https://www.act.org/content/dam/act/unsecured/documents/ACT-SAT-Concordance-Tables.pdf}
}
\usage{
data(act_sat_scale)
}
\description{
Data frame for converting between ACT and SAT scores. A range of SAT scores
converts to a single ACT score; an ACT score converts to a single
value equivalent SAT score.
}
\seealso{
Other scales:
\code{\link{grade_scale}}
}
\concept{scales}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{fye_proxy}
\alias{fye_proxy}
\title{Starting program proxies for FYE students}
\format{
\code{data.table} with 4623 rows and 2 columns keyed by student ID:
\describe{
\item{\code{mcid}}{Character, de-identified student ID. Key column.}
\item{\code{proxy}}{Character, 6-digit CIP code of the estimated
proxy program.}
}
}
\usage{
data(fye_proxy)
}
\description{
Proxies are the degree-granting engineering programs we estimate that
First-Year Engineering (FYE) students would have declared had they not been
required to enroll in FYE. Keyed by student ID. Proxies are provided for all
students in the midfielddata practice data who enroll in FYE in their first
term.
}
\details{
The proxy variable contains 6-digit CIP codes of degree-granting engineering
programs, e.g., Electrical Engineering, Mechanical Engineering, etc., that
are substituted for the FYE CIP code when an analysis requires
degree-granting starting programs. The most common application is a
graduation rate calculation.

The estimation is based on students' first post-FYE programs and a multiple
imputation suitable for categorical variables using the mice package. The
predictor variables are institution, race, and sex. The estimated variable is
the 6-digit CIP code of a degree-granting engineering program at their
institution.

\code{fye_proxy} holds only for the practice data in midfielddata---these values
cannot be commingled with the MIDFIELD research database.
}
\seealso{
Other cip-data:
\code{\link{cip}}
}
\concept{cip-data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{toy_course}
\alias{toy_course}
\title{Course data for examples}
\format{
\code{data.table} with 4616 rows and 6 columns keyed by student ID:
\describe{
\item{\code{mcid}}{Character, de-identified student ID. Key column.}
\item{\code{institution}}{Character, de-identified institution
name, e.g., Institution A, Institution B, etc.}
\item{\code{term}}{Character, academic year and term, format YYYYT.
Key column.}
\item{\code{abbrev}}{Character, course alphabetical identifier,
e.g. ENGR, MATH, ENGL. Key column.}
\item{\code{number}}{Character, course numeric identifier, e.g.
101, 3429. Key column.}
\item{\code{grade}}{Character, course grade, e.g., A+, A, A-, B+,
I, NG, etc.}
}
}
\usage{
data(toy_course)
}
\description{
Selected variables modeled on those in the \code{course} practice data for use in
package examples and articles. Sampled from an early version of the practice
data, the toy data are not a current practice data sample.
}
\seealso{
Other toy-data:
\code{\link{toy_degree}},
\code{\link{toy_student}},
\code{\link{toy_term}}
}
\concept{toy-data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_completion_status.R
\name{add_completion_status}
\alias{add_completion_status}
\title{Determine completion status for every student}
\usage{
add_completion_status(dframe, midfield_degree = degree)
}
\arguments{
\item{dframe}{Working data frame of student-level records
to which completion-status columns are to be added. Required variables
are \code{mcid} and \code{timely_term.} See also \code{add_timely_term()}.}

\item{midfield_degree}{MIDFIELD \code{degree} data table or
equivalent with required variables \code{mcid} and \code{term_degree.}}
}
\value{
A data frame in \code{data.table} format with the following
properties: rows are preserved; columns are preserved with the exception
that columns added by the function overwrite existing columns of the
same name (if any); grouping structures are not preserved. The added
columns are:
\describe{
\item{\code{term_degree}}{Character. Term in which the first degree(s) are
completed. Encoded YYYYT. Joined from \code{midfield_degree} data table.}
\item{\code{completion_status}}{Character. Label each observation to
indicate completion status. Possible values are: "timely", indicating
completion no later than the timely completion term; "late", indicating
completion after the timely completion term; and "NA" indicating
non-completion.}
}
}
\description{
Add columns to a data frame of student-level records that indicate whether a
student completed a degree, and if so, whether their completion was timely.
}
\details{
By "completion" we mean an undergraduate earning their first baccalaureate
degree (or degrees, for students earning more than one degree in the same
term). Additional degrees, if any, earned later than the term of the first
degree are ignored.

In many studies, students must complete a degree in a specified time span,
for example 4-, 6-, or 8-years after admission. If they do, their completion
is timely; if not, their completion is late and they are grouped with the
non-completers when computing a metric such as graduation rate.

Completion status is "timely" for students completing a degree no later than
their timely completion terms. See also \code{add_timely_term()}.
}
\examples{
# Start with an excerpt from the student data set 
dframe <- toy_student[1:10, .(mcid)]

# Timely term column is required to add completion status column
dframe <- add_timely_term(dframe, toy_term)

# Add completion status column
add_completion_status(dframe, toy_degree)

# Existing completion_status column, if any, is overwritten
dframe[, completion_status := NA_character_]
add_completion_status(dframe, toy_degree)
}
\seealso{
Other add_*:
\code{\link[=add_data_sufficiency]{add_data_sufficiency()}},
\code{\link[=add_timely_term]{add_timely_term()}}
}
\concept{add_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_fye_mice.R
\name{prep_fye_mice}
\alias{prep_fye_mice}
\title{Prepare FYE data for multiple imputation}
\usage{
prep_fye_mice(
  midfield_student = student,
  midfield_term = term,
  ...,
  fye_codes = NULL
)
}
\arguments{
\item{midfield_student}{MIDFIELD \code{student} data table or
equivalent with required variables \code{mcid}, \code{race}, and \code{sex.}}

\item{midfield_term}{MIDFIELD \code{term} data table or equivalent
with required variables \code{mcid}, \code{institution}, \code{term}, and \code{cip6.}}

\item{...}{Not used for passing values; forces subsequent arguments to be
referable only by name.}

\item{fye_codes}{Optional character vector of 6-digit CIP codes to identify
FYE programs, default "140102". Codes must be 6-digit strings of numbers;
regular expressions are prohibited. Non-engineering codes---those that do
not start with 14---produce an error.}
}
\value{
A data frame in \code{data.table} format conditioned for
later use as an input to the mice R package for multiple imputation. The
data frame comprises one row for every FYE student, first-term and
migrator. Grouping structures are not preserved. The columns returned
are:
\describe{
\item{\code{mcid}}{Character, anonymized student identifier. Returned
as-is.}
\item{\code{race}}{Factor, race/ethnicity as self-reported by the
student. An imputation predictor variable.}
\item{\code{sex}}{Factor, sex as self-reported by the
student. An imputation predictor variable.}
\item{\code{institution}}{Factor, anonymized institution name. An
imputation predictor variable.}
\item{\code{proxy}}{Factor, 6-digit CIP code of a student's known,
post-FYE engineering program or NA representing missing
values to be imputed.}
}
}
\description{
Constructs a data frame of student-level records of First-Year Engineering
(FYE) programs and conditions the data for later use as an input to the mice
R package for multiple imputation. Sets up three variables as predictors
(institution, race/ethnicity, and sex) and one variable to be estimated
(program CIP code).
}
\details{
At some US institutions, engineering students are required to complete a
First-Year Engineering (FYE) program as a prerequisite for declaring an
engineering major. Administratively, degree-granting engineering programs
such as Electrical Engineering or Mechanical Engineering treat their incoming
post-FYE students as their "starting" cohorts.  However, when computing a
metric that requires a count of starters---graduation rate, for example---FYE
records must be treated with special care to avoid a miscount.

To illustrate the potential for miscounting starters, suppose we wish to
calculate a Mechanical Engineering (ME) graduation rate. Students starting in
ME constitute the starting pool and the fraction of that pool graduating in
ME is the graduation rate. At FYE institutions, an ME program would typically
define their starting pool as the post-FYE cohort entering their program.
This may be the best information available, but it invariably undercounts
starters by failing to account for FYE students who do not transition
(post-FYE) to degree-granting engineering programs---students who may have
left the institution or switched to non-engineering majors. In either case,
in the absence of the FYE requirement, some of these students would have been
ME starters. By neglecting these students, the count of ME starters is
artificially low resulting in an ME graduation rate that is artificially
high. The same is true for every degree-granting engineering discipline in an
FYE institution.

Therefore, to avoid miscounting starters at FYE institutions, we have to
estimate an "FYE proxy", that is, the 6-digit CIP codes of the
degree-granting engineering programs that FYE students would have declared
had they not been required to enroll in FYE. The purpose of
`prep_fye_mice()`` is to prepare the data for making that estimation.

After running \code{prep_fye_mice()} but before running \code{mice()}, one can edit
variables or add variables to create a custom set of predictors. The mice
package expects all predictors and the proxy variables to be factors. Do not
delete the institution variable because it ensures that a student's imputed
program is available at their institution.

In addition, ensure that the only missing values are in the proxy column.
Other variables are expected to be complete (no NA values). A value of
"unknown" in a predictor column, e.g., race/ethnicity or sex, is an
acceptable value, not missing data. Observations with missing or unknown
values in the ID or institution columns (if any) should be removed.
}
\section{Method}{
 The function extracts all terms for all FYE students,
including those who migrate to enter Engineering after their first term,
and identifies the first post-FYE program in which they enroll, if any.
This treatment yields two possible outcomes for values returned in the
\code{proxy} column:

\enumerate{
\item{The student completes FYE and enrolls in an engineering major. For
this outcome, we know that at the student's first opportunity, they
enrolled in an engineering program of their choosing. The CIP code of
that program is returned as the student's FYE proxy.}
\item{The student does not enroll post-FYE in an engineering major. Such
students have no further records in the database or switched from
Engineering to another program. For this outcome, the data provide no
information regarding what engineering program the student would have
declared originally had the institution not required them to enroll in
FYE. For these students a proxy value of NA is returned. These are the
data treated as missing values to be imputed by \code{mice()}.}
}

In cases where students enter FYE, change programs, and re-enter FYE, only
the first group of FYE terms is considered. Any programs before FYE are
ignored.

The resulting data frame is ready for use as input for the mice package,
with all variables except \code{mcid} returned as factors.
}

\examples{
# Using toy data
prep_fye_mice(toy_student, toy_term)

# Other columns, if any, are dropped
colnames(toy_student)
colnames(prep_fye_mice(toy_student, toy_term))

# Optional argument permits multiple CIP codes for FYE
prep_fye_mice(midfield_student = toy_student, 
              midfield_term =toy_term, 
              fye_codes = c("140101", "140102"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/midfieldr-package.R
\docType{package}
\name{midfieldr-package}
\alias{midfieldr}
\alias{midfieldr-package}
\title{midfieldr: Tools and Methods for Working with MIDFIELD Data in 'R'}
\description{
\if{html}{\figure{logo.svg}{options: style='float: right' alt='logo' width='120'}}

Provides tools and demonstrates methods for working with individual undergraduate student-level records (registrar's data) in 'R'. Tools include filters for program codes, data sufficiency, and timely completion. Methods include gathering blocs of records, computing quantitative metrics such as graduation rate, and creating charts to visualize comparisons. 'midfieldr' interacts with practice data provided in 'midfielddata', an R data package available at \url{https://midfieldr.github.io/midfielddata/}. 'midfieldr' also interacts with the full MIDFIELD database for users who have access. As of the transfer of MIDFIELD to the American Society for Engineering Education in 2023, the development, expansion, and study of MIDFIELD has been supported by the National Science Foundation grants 0337629, 0646441, 0729596, 0734062, 0835914, 0935157, 0935058, 0969474, 1025171, 1129383, 1232740, 1329283, 1361058, 1545667, 2142087, 2141903, and 2152441.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://midfieldr.github.io/midfieldr/}
  \item Report bugs at \url{https://github.com/MIDFIELDR/midfieldr/issues}
}

}
\author{
\strong{Maintainer}: Richard Layton \email{graphdoctor@gmail.com} [copyright holder]

Authors:
\itemize{
  \item Richard Layton \email{graphdoctor@gmail.com} [copyright holder]
  \item Russell Long \email{ralong@purdue.edu} [copyright holder, data manager]
  \item Matthew Ohland \email{ohland@purdue.edu} [copyright holder]
  \item Marisa Orr [copyright holder]
  \item Susan Lord \email{slord@sandiego.edu} [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{study_programs}
\alias{study_programs}
\title{Case-study program labels and codes}
\format{
\code{data.table} with 15 rows and 2 columns.
The variables are:
\describe{
\item{\code{cip6}}{Character, 6-digit CIP code of program in which
a student is enrolled in a term.}
\item{\code{program}}{Character, abbreviated labels for four engineering
programs. Values are "CE" (Civil Engineering), "EE" (Electrical
Engineering), "ISE" (Industrial/Systems Engineering), and  "ME" (Mechanical
Engineering).}
}
}
\usage{
data(study_programs)
}
\description{
Data table of program CIP codes and labels of the four programs of the case
study. Keyed by 6-digit CIPs. Provided for the convenience of vignette users.
}
\details{
Starting with the midfieldr \code{cip} data set, we extracted the CIPs of the four
programs of the case study and assigned them a custom label to be used for
grouping and summarizing.
}
\seealso{
Other case-study-data:
\code{\link{baseline_mcid}},
\code{\link{study_observations}},
\code{\link{study_results}}
}
\concept{case-study-data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{toy_degree}
\alias{toy_degree}
\title{Degree data for examples}
\format{
\code{data.table} with 65 rows and 4 columns keyed by student ID.
The variables are:
\describe{
\item{\code{mcid}}{Character, de-identified student ID. Key column.}
\item{\code{institution}}{Character, de-identified institution
name, e.g., Institution A, Institution B, etc.}
\item{\code{term_degree}}{Character, academic year and term
in which a student completes their program, format YYYYT.}
\item{\code{cip6}}{Character, 6-digit CIP code of program in which
a student is enrolled in a term.}
}
}
\usage{
data(toy_degree)
}
\description{
Selected variables modeled on those in the \code{degree} practice data for use in
package examples and articles. Sampled from an early version of the practice
data, the toy data are not a current practice data sample.
}
\seealso{
Other toy-data:
\code{\link{toy_course}},
\code{\link{toy_student}},
\code{\link{toy_term}}
}
\concept{toy-data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{study_observations}
\alias{study_observations}
\title{Case-study observations}
\format{
\code{data.table} with 8917 rows and 5 columns.
The variables are:
\describe{
\item{\code{mcid}}{Character, de-identified student ID. Key column.}
\item{\code{race}}{Character, race/ethnicity as self-reported
by the student, e.g., Asian, Black, Hispanic, etc.}
\item{\code{sex}}{Character, sex as self-reported by the student,
possible values are Female, Male, and Unknown.}
\item{\code{program}}{Character, academic program label.}
\item{\code{bloc}}{Character, indicating the grouping
(\code{ever_enrolled} or \code{graduates}) to which an observation
belongs.}
}
}
\usage{
data(study_observations)
}
\description{
Data table of post-processed observations of students ever enrolled in, and
students graduating from, the four programs of the case study. Keyed by
student ID. Provided for the convenience of vignette users.
}
\details{
Starting with the case-study starting pool of students ever enrolled in the
four programs of the study (Civil, Electrical, Industrial/Systems, and
Mechanical Engineering), we filtered the data for data sufficiency, degree
seeking, program, and timely completion.

A data frame of "ever enrolled" and a data frame of "timely graduates" were
bound using shared column names and are distinguished in the \code{bloc} variable.
This data structure facilitates grouping and summarizing by race, sex,
program, and group.
}
\seealso{
Other case-study-data:
\code{\link{baseline_mcid}},
\code{\link{study_programs}},
\code{\link{study_results}}
}
\concept{case-study-data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/order_multiway.R
\name{order_multiway}
\alias{order_multiway}
\title{Order categorical variables of multiway data}
\usage{
order_multiway(
  dframe,
  quantity,
  categories,
  ...,
  method = NULL,
  ratio_of = NULL
)
}
\arguments{
\item{dframe}{Data frame containing a single quantitative value
(or response) for every combination of levels of two categorical
variables. Categories may be class character or factor. Two additional
numeric columns are required when using the "percent" ordering method.}

\item{quantity}{Character, name (in quotes) of the single multiway quantitative
variable}

\item{categories}{Character, vector of names (in quotes) of the two multiway
categorical variables}

\item{...}{Not used for passing values; forces subsequent arguments to be
referable only by name.}

\item{method}{Character, “median” (default) or “percent”, method of
ordering the levels of the categories. The median method computes the
medians of the quantitative column grouped by category. The percent
method computes percentages based on the same ratio underlying the
quantitative percentage variable except grouped by category.}

\item{ratio_of}{Character vector with the names (in quotes) of the
numerator and denominator columns that produced the quantitative
variable, required when \code{method} is "percent". Names can be
in any order; the algorithm assumes that the parameter with the
larger column sum is the denominator of the ratio.}
}
\value{
A data frame in \code{data.table} format with
the following properties: rows are preserved; columns specified by
\code{categories} are converted to factors and ordered; the column specified
by \code{quantity} is converted to type double; other columns are preserved
with the exception that columns added by the function overwrite existing
columns of the same name (if any); grouping structures are not preserved.
The added columns are:
\describe{
\item{\code{CATEGORY_median} columns (when ordering method is "median")}{
Numeric. Two columns of medians of the quantitative variable grouped
by the categorical variables. The \code{CATEGORY} placeholder in
the column name is replaced by a category name from the
\code{categories} argument. For example, suppose
\code{categories = c("program", "people")} and
\code{method = "median"}. The two new column names would be
\code{program_median} and \code{people_median.}}

\item{\code{CATEGORY_QUANTITY} columns (when ordering method is "percent")}{
Numeric. Two columns of percentages based on the same ratio that
produces the quantitative variable except grouped by the categorical
variables. The \code{CATEGORY} placeholder in the column name is
replaced by a category name from the \code{categories} argument; the
\code{QUANTITY} placeholder is replaced by the quantitative variable
name in the \code{quantity} argument. For example, suppose
\code{categories = c("program", "people")}, and
\code{quantity = "grad_rate"}, and \code{method = "percent"}. The two
new column names  would be \code{program_grad_rate} and
\code{people_grad_rate.}}

}
}
\description{
Transform a data frame such that two independent categorical variables are
factors with levels ordered for display in a multiway dot plot. Multiway data
comprise a single quantitative value (or response) for every combination of
levels of two categorical variables. The ordering of the rows and panels is
crucial to the perception of effects (Cleveland, 1993).
}
\details{
In our context, "multiway" refers to the data structure and graph design
defined by Cleveland (1993), not to the methods of analysis described by
Kroonenberg (2008).

Multiway data comprise three variables: a categorical variable of \emph{m} levels;
a second independent categorical variable of \emph{n} levels; and a quantitative
variable (or \emph{response}) of length \emph{mn} that cross-classifies the categories,
that is, there is a value of the response for each combination of levels of
the two categorical variables.

In a multiway dot plot, one category is encoded by the panels, the second
category is encoded by the rows of each panel, and the quantitative variable
is encoded along identical horizontal scales.
}
\examples{
# Subset of built-in data set
dframe <- study_results[program == "EE" | program == "ME"]
dframe[, people := paste(race, sex)]
dframe[, c("race", "sex") := NULL]
data.table::setcolorder(dframe, c("program", "people"))

# Class before ordering
class(dframe$program)
class(dframe$people)

# Class and levels after ordering
mw1 <- order_multiway(dframe, 
                      quantity = "stickiness", 
                      categories = c("program", "people"))
class(mw1$program)
levels(mw1$program)
class(mw1$people)
levels(mw1$people)

# Display category medians 
mw1

# Existing factors (if any) are re-ordered
mw2 <- dframe
mw2$program <- factor(mw2$program, levels = c("ME", "EE"))

# Levels before conditioning
levels(mw2$program) 

# Levels after conditioning
mw2 <- order_multiway(dframe, 
                      quantity = "stickiness", 
                      categories = c("program", "people"))
levels(mw2$program) 

# Ordering using percent method
order_multiway(dframe, 
               quantity = "stickiness", 
               categories = c("program", "people"), 
               method = "percent", 
               ratio_of = c("graduates", "ever_enrolled"))
}
\references{
Cleveland WS (1993). \emph{Visualizing Data}. Hobart Press, Summit, NJ.

Kroonenberg PM (2008). \emph{Applied Multiway Data Analysis}. Wiley,
Hoboken, NJ.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select_required.R
\name{select_required}
\alias{select_required}
\title{Select required midfieldr variables}
\usage{
select_required(midfield_x, ..., select_add = NULL)
}
\arguments{
\item{midfield_x}{Data frame from which columns are selected.}

\item{...}{Not used for passing values; forces subsequent arguments to be
referable only by name.}

\item{select_add}{Character vector of additional column names to return.}
}
\value{
A data frame in \code{data.table} format with the
following properties: rows are preserved; columns with names that
match or partially match values in the default set plus any in
\code{select_add} are retained; grouping structures are not preserved.
}
\description{
Subset a data frame, selecting columns by matching or partially matching a
vector of character strings. A convenience function to reduce the dimensions
of a MIDFIELD data table at the start of a session by selecting only those
columns required by other midfieldr functions or that are required to form a
composite key. Particularly useful in interactive sessions when viewing the
data tables at various stages of an analysis.
}
\details{
Several midfieldr functions require one or more of the variables \code{mcid},
\code{institution}, \code{race}, \code{sex}, \verb{^term}, \code{cip6}, and \code{level}. And if
one requires a composite key to uniquely identify rows in the course or
degree tables, the variables \code{abbrev}, \code{number} and degree variable
\code{degree} are also required. A vector of these names comprises the default
subset.

Additional column names or partial names can be included by using the
\code{select_add} argument.

The column names of \code{midfield_x} are searched for matches or partial matches
using \code{grep()}, thus search terms can include regular expressions. Variables
with names that match or partially match the search terms are returned; all
other columns are dropped. Rows are unaffected. Search terms not present are
silently ignored.

One could use this function to select columns from a non-MIDFIELD data frame,
but with no benefit to the user---conventional column selection syntax is
better suited to that task. Here, we specialize the column selection to serve
midfieldr functions.
}
\examples{
# Default character vector for selecting columns
default_cols<- c("mcid", "institution", "race", "sex", "^term", "cip6", "level")

# Create one string separated by OR
search_pattern <- paste(default_cols, collapse = "|")

# Find names of columns matching or partially matching 
x <- select_required(toy_student) 
names(x)
grepl(search_pattern, names(x))

x <- select_required(toy_term) 
names(x)
grepl(search_pattern, names(x))

x <- select_required(toy_degree) 
names(x)
grepl(search_pattern, names(x))

x <- select_required(toy_course) 
names(x)
grepl(search_pattern, names(x))

# Adding search terms
x <- select_required(toy_course, select_add = c("abbrev", "number", "grade")) 
names(x)



}
\concept{select_*}
